local addon,L = DXE,DXE.L
local EDB = addon.EDB
local module = addon:NewModule("Options")
addon.Options = module

local db
local opts
local opts_args
local match,gmatch,gsub,find,split,sub = string.match,string.gmatch,string.gsub,string.find,string.split,string.sub

local DEFAULT_WIDTH = 960
local DEFAULT_HEIGHT = 700

local AC = LibStub("AceConfig-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local SM = LibStub("LibSharedMedia-3.0")

local class_color = {}
for class,color in pairs(RAID_CLASS_COLORS) do
	class_color[class] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end
local function getclasscolor(unit)
	if unit == nil then return "nil" end
	local classc = select(2,UnitClass(unit))
	if classc == nil and unit ~= nil then return unit end --- NPCs, BOSSs
	return class_color[classc]..unit.."|r" or unit
end
local function genblank(order) return {type = "description", name = "", order = order} end

local function InitializeOptions()
	opts = {
		type = "group",
		name = "DXE",
		handler = addon,
		disabled = function() return not db.profile.Enabled end,
		args = {
			dxe_header = {
				type = "header",
				name = format("%s - %s",L.loader["Deus Vox Encounters"],L.options["Version"])..format(": |cff99ff33%d|r",addon.version),
				order = 1,
				width = "full",
			},
			Enabled = {
				type = "toggle",
				order = 100,
				name = L.options["Enabled"],
				get = "IsEnabled",
				width = "half",
				set = function(info,val) db.profile.Enabled = val
						if val then addon:Enable()
						else addon:Disable() end
				end,
				disabled = function() return false end,
			},
			ToggleLock = {
				type = "execute",
				order = 200,
				name = L.options["Toggle Anchors"],
				desc = L.options["Toggle frame anchors. You can also toggle this by clicking on the pane's pad lock icon"],
				func = function() addon:ToggleLock() end,
			},
		},
	}

	module.opts = opts

	opts_args = opts.args

	-- Minimap
	if LibStub("LibDBIcon-1.0",true) then
		local LDBIcon = LibStub("LibDBIcon-1.0")
		opts_args.ShowMinimap = {
			type = "toggle",
			order = 150,
			name = L.options["Minimap"],
			desc = L.options["Show minimap icon"],
			get = function() return not DXEIconDB.hide end,
			set = function(info,v) DXEIconDB.hide = not v; LDBIcon[DXEIconDB.hide and "Hide" or "Show"](LDBIcon,"DXE") end,
			width = "half",
		}
	end

	---------------------------------------------
	-- UTILITY
	---------------------------------------------

	local GetSounds
	do
		local sounds = {}
		function GetSounds()
			table.wipe(sounds)
			for id,name in pairs(db.profile.Sounds) do
				if id:find("^ALERT") then sounds[id] = id end
			end
			for id,name in pairs(db.profile.CustomSounds) do
				sounds[id] = id
			end
			sounds["None"] = L.options["None"]
			return sounds
		end
		
	end

	---------------------------------------------
	-- GENERAL
	---------------------------------------------

	do
		local colors = {}
		for k,c in pairs(addon.Media.Colors) do
			local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
			colors[k] = hex
		end
		local globals_group = {
			type = "group",
			name = L.options["Globals"],
			order = 50,
			get = function(info)
				local var = info[#info]
				if var:find("Color") then return unpack(db.profile.Globals[var])
				else return db.profile.Globals[var] end
			end,
			set = function(info,v,v2,v3,v4)
				local var = info[#info]
				if var:find("Color") then
					local t = db.profile.Globals[var]
					t[1],t[2],t[3],t[4] = v,v2,v3,v4
					addon["Notify"..var.."Changed"](addon,v,v2,v3,v4)
				else
					db.profile.Globals[var] = v
					addon:RefreshProfile()
					addon.Alerts:RefreshProfile()
					addon["Notify"..var.."Changed"](addon,v)
				end
			end,
			args = {
				header = {
					type = "group",
					order = 1305,
					name = L.options["Settings"],
					inline = true,
					args = {				
						BarTexture = {
							type = "select",
							order = 1,
							name = L.options["Bar Texture"],
							desc = L.options["Bar texture used throughout the addon"],
							values = SM:HashTable("statusbar"),
							dialogControl = "LSM30_Statusbar",
						},
						BackgroundTexture = {
							order = 2,
							type = "select",
							name = L.options["Background Texture"],
							desc = L.options["Background texture used throughout the addon"],
							values = SM:HashTable("background"),
							dialogControl = "LSM30_Background",
						},
						BackgroundInset = {
							order = 3,
							type = "range",
							name = L.options["Background Inset"],
							desc = L.options["How far in or out the background is from the edges"],
							min = -20,
							max = 20,
							step = 0.1,
						},
						Border = {
							order = 4,
							type = "select",
							name = L.options["Border"],
							desc = L.options["Border used throughout the addon"],
							values = SM:HashTable("border"),
							dialogControl = "LSM30_Border",
						},
						BorderEdgeSize = {
							order = 5,
							type = "range",
							name = L.options["Border Edge Size"],
							desc = L.options["Border size used througout the addon"],
							min = 0.1,
							max = 20,
							step = 0.1,
						},
						Font = {
							order = 11,
							type = "select",
							name = L.options["Main Font"],
							desc = L.options["Font used throughout the addon"],
							values = SM:HashTable("font"),
							dialogControl = "LSM30_Font",
						},
						TimerFont = {
							order = 12,
							type = "select",
							name = L.options["Timer Font"],
							desc = L.options["Font used for timers"],
							values = SM:HashTable("font"),
							dialogControl = "LSM30_Font",
						},
						misc_header = {
							type = "header",
							name = L.options["Global Color"],--Miscellaneous
							order = 13,
						},	
						BorderColor = {
							order = 14,
							type = "color",
							name = L.options["Border Color"],
							desc = L.options["Border color used throughout the addon"],
							hasAlpha = true,
						},
						BackgroundColor = {
							order = 15,
							type = "color",
							name = L.options["Background Color"],
							desc = L.options["Background color used throughout the addon"],
							hasAlpha = true,
						},
						GlobalBarCrl = {
							order = 16,
							type = "select",
							name = L.options["Bar Color"],
							desc = L.options["Bar color used throughout the addon"],
							values = colors,
						},
						test = {
							type = "execute",
							name = L.options["Test"],
							order = 18,
							func = function()
								local c = db.profile.Globals.GlobalBarCrl
								addon.Alerts:GlobalTest(c)
							end,
						},				
					},
				},

				header2 = {
					type = "group",
					order = 1310,
					name = L.options["Here you can configure settings related to boss kills/wipes chat messages"],
					inline = true,
					args = {				
					blank2 = genblank(2.1),
						--[[tt_anchor_header = {
							type = "header",
							name = L.options["Settings"],
							order = 3,
						},--]]
						ShowKill = {
							order = 4,
							type = "toggle",
							name = L.options["Show Kill"],
							width = "normal",
							get = function(info)
								return db.profile.Kills.ShowKill
							end,
							set = function(info,v)
								db.profile.Kills.ShowKill = v
							end,		
							disabled = function() return not db.profile.Kills.KEnabled end,						
						},
						ShowBestKill = {
							order = 5,
							type = "toggle",
							name = L.options["Show Best Kill"],
							width = "normal",
							get = function(info)
								return db.profile.Kills.ShowBestKill
							end,
							set = function(info,v)
								db.profile.Kills.ShowBestKill = v
							end,	
							disabled = function() return not db.profile.Kills.KEnabled end,						
						},
						ShowKillCount = {
							order = 6,
							type = "toggle",
							name = L.options["Show Kill Count"],
							width = "double",
							get = function(info)
								return db.profile.Kills.ShowKillCount
							end,
							set = function(info,v)
								db.profile.Kills.ShowKillCount = v
							end,
							disabled = function() return not db.profile.Kills.KEnabled end,						
						},
						ShowWipe = {
							order = 7,
							type = "toggle",
							name = L.options["Show Wipe"],
							width = "normal",
							get = function(info)
								return db.profile.Kills.ShowWipe
							end,
							set = function(info,v)
								db.profile.Kills.ShowWipe = v
							end,
							disabled = function() return not db.profile.Kills.KEnabled end,						
						},
						ShowWipeCount = {
							order = 8,
							type = "toggle",
							name = L.options["Show Wipe Count"],
							width = "normal",
							get = function(info)
								return db.profile.Kills.ShowWipeCount
							end,
							set = function(info,v)
								db.profile.Kills.ShowWipeCount = v
							end,
							disabled = function() return not db.profile.Kills.KEnabled end,						
						},	
					},
				},
			},
		}

		opts_args.globals_group = globals_group

	end
	
	---------------------------------------------
	-- PANE
	---------------------------------------------

	do
		local pane_group = {
			type = "group",
			name = L.options["Pane"],
			order = 100,
			get = function(info)
				local var = info[#info]
				if var:find("Color") then return unpack(db.profile.Pane[var])
				else return db.profile.Pane[var] end end,
			set = function(info,v) db.profile.Pane[info[#info]] = v end,
			args = {}
		}
		opts_args.pane_group = pane_group

		local pane_args = pane_group.args

		do
			local visibility_group = {
				type = "group",
				name = "",
				inline = true,
				order = 100,
				set = function(info,v)
					db.profile.Pane[info[#info]] = v
					addon:UpdatePaneVisibility()
				end,
				disabled = function() return not db.profile.Pane.Show end,
				args = {
					Show = {
						order = 100,
						type = "toggle",
						name = L.options["Show Pane"],
						desc = L.options["Toggle the visibility of the pane"],
						disabled = function() return false end,
					},
					showpane_desc = {
						order = 150,
						type = "description",
						name = L.options["Show Pane"].."...",
					},
					OnlyInRaid = {
						order = 200,
						type = "toggle",
						name = L.options["Only in raids"],
						desc = L.options["Show the pane only in raids"],
						width = "full",
					},
					OnlyInParty = {
						order = 210,
						type = "toggle",
						name = L.options["Only in party"],
						desc = L.options["Show the pane only in party"],
						width = "full",
					},
					OnlyInRaidInstance = {
						order = 250,
						type = "toggle",
						name = L.options["Only in raid instances"],
						desc = L.options["Show the pane only in raid instances"],
						width = "full",
					},
					OnlyInPartyInstance = {
						order = 255,
						type = "toggle",
						name = L.options["Only in party instances"],
						desc = L.options["Show the pane only in party instances"],
						width = "full",
					},
					OnlyIfRunning = {
						order = 260,
						type = "toggle",
						name = L.options["Only if engaged"],
						desc = L.options["Show the pane only if an encounter is running"],
						width = "full",
					},
					OnlyOnMouseover = {
						order = 261,
						type = "toggle",
						name = L.options["Only on mouseover"],
						desc = L.options["Show the pane only if the mouse is over it"],
						width = "full",
					},
				},
			}

			pane_args.visibility_group = visibility_group

			local skin_group = {
				type = "group",
				name = "",
				order = 200,
				inline = true,
				set = function(info,v,v2,v3,v4)
					local var = info[#info]
					if var:find("Color") then db.profile.Pane[var] = {v,v2,v3,v4}
					else db.profile.Pane[var] = v end
					addon:SkinPane()
				end,
				args = {
					BarGrowth = {
						order = 200,
						type = "select",
						name = L.options["Bar Growth"],
						desc = L.options["Direction health watcher bars grow. If set to automatic, they grow based on where the pane is"],
						values = {AUTOMATIC = L.options["Automatic"], UP = L.options["Up"], DOWN = L.options["Down"]},
						set = function(info,v)
							db.profile.Pane.BarGrowth = v
							addon:LayoutHealthWatchers()
						end,
						disabled = function() return not db.profile.Pane.Show end,
					},
					Scale = {
						order = 300,
						type = "range",
						name = L.options["Scale"],
						desc = L.options["Adjust the scale of the pane"],
						set = function(info,v)
							db.profile.Pane.Scale = v
							addon:ScalePaneAndCenter()
						end,
						min = 0.1,
						max = 2,
						step = 0.1,
					},
					Width = {
						order = 310,
						type = "range",
						name = L.options["Width"],
						desc = L.options["Adjust the width of the pane"],
						set = function(info, v)
							db.profile.Pane.Width = v
							addon:SetPaneWidth()
						end,
						min = 175,
						max = 500,
						step = 1,
					},
					BarSpacing = {
						order = 320,
						type = "range",
						name = L.options["Bar Spacing"],
						desc = L.options["How far apart health bars are"],
						set = function(info, v)
							db.profile.Pane.BarSpacing = v
							addon:LayoutHealthWatchers()
						end,
						min = 0,
						max = 50,
						step = 0.1,
					},
					font_header = {
						type = "header",
						name = L.options["Font"],
						order = 750,
					},
					TitleFontSize = {
						order = 900,
						type = "range",
						name = L.options["Title Font Size"],
						desc = L.options["Select a font size used on health watchers"],
						min = 8,
						max = 20,
						step = 1,
					},
					HealthFontSize = {
						order = 1000,
						type = "range",
						name = L.options["Health Font Size"],
						desc = L.options["Select a font size used on health watchers"],
						min = 8,
						max = 20,
						step = 1,
					},
					FontColor = {
						order = 1100,
						type = "color",
						name = L.options["Font Color"],
						desc = L.options["Set a font color used on health watchers"],
					},
					misc_header = {
						type = "header",
						name = L.options["Miscellaneous"],
						order = 1200,
					},
					NeutralColor = {
						order = 1300,
						type = "color",
						name = L.options["Neutral Color"],
						desc = L.options["The color of the health bar when first shown"],
					},
					LostColor = {
						order = 1400,
						type = "color",
						name = L.options["Lost Color"],
						desc = L.options["The color of the health bar after losing the mob"],
					},
					blan = genblank(2200.1),
					blan2 = genblank(2200.2),
					PixelPerfect_header = {
						type = "header",
						name = L.options["Pixel Perfect"],
						order = 2300,
					},
					blank = genblank(2400),
					blank2 = genblank(2400.1),
					PaneAnchor_x = {
						type = "range",
						name = L.options["X Position"],
						min = -2048,
						max = 2048,
						step = 1,
						order = 2500,
						width = "full",
						get = function(info) 
							local pos = db.profile.Positions["DXEPane"]
							if pos then
								return pos.xOfs
							else
								return 179.0000387755648
							end
						end,
						set = function(info,v)
							db.profile.Positions["DXEPane"].xOfs = v
							addon:LoadPosition("DXEPane")
						end,							
					},
					PaneAnchor_y = {
						type = "range",
						name = L.options["Y Position"],
						min = -2048,
						max = 2048,
						step = 1,
						order = 2600,
						width = "full",
						get = function(info)
							local pos = db.profile.Positions["DXEPane"]
							if pos then
								return pos.yOfs
							else
								return -158.9998021557163
							end
						end,
						set = function(info,v)
							db.profile.Positions["DXEPane"].yOfs = v
							addon:LoadPosition("DXEPane")
						end,
					},					
				},
			}

			pane_args.skin_group = skin_group
		end
	end

	---------------------------------------------
	-- MISCELLANEOUS
	---------------------------------------------
	do
		local misc_group = {
			type = "group",
			name = L.options["Miscellaneous"],
			get = function(info) return db.profile.Misc[info[#info]] end,
			set = function(info,v) db.profile.Misc[info[#info]] = v end,
			order = 300,
			args = {
				--[[header1 = {
					type = "header",
					name = L.options["Boss Mods"],
					order = 50,
				},--]]	
				timertext_group = {
					type = "group",
					name = L.options["Boss Mods"],
					order = 1,
					inline = true,
					args = {				
						BlockRaidWarningFrame = {
							type = "toggle",
							name = L.options["Block raid warning frame messages from other boss mods"],
							order = 100,
							width = "full",
						},
						BlockRaidWarningMessages = {
							type = "toggle",
							name = L.options["Block raid warning messages, in the chat log, from other boss mods"],
							order = 200,
							width = "full",
						},
					},
				},
				--[[header2 = {
					type = "header",
					name = L.options["Boss Emotes"],
					order = 250,
				},--]]
				header2_group = {
					type = "group",
					name = L.options["Boss Emotes"],
					order = 2,
					inline = true,
					args = {				
						BlockBossEmoteFrame = {
							type = "toggle",
							name = L.options["Block boss emote frame messages"],
							order = 300,
							width = "full",
						},
						BlockBossEmoteMessages = {
							type = "toggle",
							name = L.options["Block boss emote messages in the chat log"],
							order = 400,
							width = "full",
						},
					},
				},	
				--[[header3 = {
					type = "header",
					name = L.options["Movie Filter"],
					order = 500,
				},--]]
				header3_group = {
					type = "group",
					name = L.options["Movie Filter"],
					order = 3,
					inline = true,
					args = {					
						BlockMovies = {
							type = "toggle",
							name = L.options["Block raid movies after you've seen them once."],
							order = 700,
							width = "full",
						},
					},
				},
				header4_group = {
					type = "group",
					name = L.options["Miscellaneous"],
					order = 4,
					inline = true,
					args = {					
						BlockGarrison = {
							type = "toggle",
							name = L.options["Block garrison alerts"],
							order = 700,
							width = "full",
						},
						BlockGuildChallenge = {
							type = "toggle",
							name = L.options["Block guild challenge alerts"],
							order = 701,
							width = "full",
						},						
					},
				},				
			},
		}
		opts_args.misc_group = misc_group
	end

	---------------------------------------------
	-- Lag Tolerance
	---------------------------------------------
	do
		local LagTolerance = addon.LagTolerance
		local LT_group = {
			type = "group",
			name = L.options["Lag Tolerance"],
			get = function(info) return LagTolerance.db.profile[info[#info]] end,
			set = function(info,v) LagTolerance.db.profile[info[#info]] = v end,
			order = 400,
			args = {
				header = {
					type = "header",
					name = L.options["Lag Tolerance"],
					order = 1200,
				},
				desc = {
					type = "description",
					name = L.options["Determines how far ahead of the 'end of a spell' (Global Cooldown) start-recovery spell system can be, before allowing spell request to be sent to the server. This controls the built-in lag for the ability queuing system wich can be set in Options > Combat > Custom Lag Tolerance. Ideally, you'll want to set this to your in-game latency."],
					order = 1300,
					width = "full",
				},
				blank1 = genblank(1301),
				blank2 = genblank(1302),
				desc2 = {
					type = "description",
					name = L.options["Because your Latency is always changing when fighting, Lag Tolerance matches your latency to provide the best 'end of a spell' possible, Blizzard limits this to be no sooner than 30 seconds."],
					order = 1303,
					width = "full",
				},
				--[[header3 = {
					type = "header",
					name = L.options["Settings"],
					order = 1304,
				},--]]
				blank3 = genblank(1304),
				header3 = {
					type = "group",
					order = 1305,
					name = L.options["Settings"],
					inline = true,
					args = {				
						enabled = {
							type = "toggle",
							name = L.options["Enable Lag Tolerance"],
							order = 1400,
							width = "normal",
						},
						Interval = {
							name = L.options["Refresh every X seconds"],
							type = "range",
							min = 30, max = 60, step = 5,
							order = 1401,
							--arg = "refreshEvery"
						},
						Offset = {
							name = L.options["Change Final Value"],
							desc = L.options["By default Lag Tolerance will be exact match to your current ping, this setting lets you tweak the final value."],
							type = "range",
							min = -20, max = 20, step = 1,
							order = 1402,
						--	arg = "refreshEvery"
						},
					},
				},
			},
		}

		opts_args.LT_group = LT_group
	end
	
	---------------------------------------------
	-- Auto Responder
	---------------------------------------------
	do
		local AutoResponder = addon.AutoResponder
		local AR_group = {
			type = "group",
			name = L.options["Auto Responder"],
			get = function(info) return AutoResponder.db.profile[info[#info]] end,
			set = function(info,v) AutoResponder.db.profile[info[#info]] = v end,
			order = 400,
			args = {
				header = {
					type = "header",
					name = L.options["Auto Responder"],
					order = 1200,
				},
				desc = {
					type = "description",
					name = L.options["Activates the automatic responder for whispers during boss encounters and receive a short summary of the current fight (boss name, time elapsed, how many raid members are alive). The auto respnder works for normal and Battle.net whispers."],
					order = 1300,
					width = "full",
				},
				blank = genblank(1301),
				header = {
					type = "group",
					order = 1305,
					name = L.options["Settings"],
					inline = true,
					args = {				
						enabled = {
							type = "toggle",
							name = L.options["Enable Auto Responder"],
							order = 1400,
							width = "double",
						},
						blank2 = genblank(1500),
						whisperfilter = {
							type = "toggle",
							name = L.options["Enable Whisper Filter"],
							desc = L.options["This option filter the message dxestatus sent to player appearing to you, disable if you want to see it."],
							order = 1600,
							width = "double",
						},	
					},
				},			
			},
		}

		opts_args.AR_group = AR_group
	end

	---------------------------------------------
	-- ABOUT
	---------------------------------------------

	do
		local about_group = {
			type = "group",
			name = L.options["About"],
			order = -2,
			args = {
				authors_desc = {
					type = "description",
					name = format("%s : %s",L.options["Original Authors"],"|cffffd200Kollektiv|r, |cffffd200Fariel|r"),
					order = 100,
				},
				--[[blank1 = genblank(150),
				created_desc = {
					type = "description",
					name = format(L.options["Created for use by %s on %s"],"|cffffd200Deus Vox|r","|cffffff78US-Laughing Skull|r"),
					order = 200,
				},--]]
				blank2 = genblank(250),
				visit_desc = {
					type = "description",
					name = format("%s: %s",L.options["Website"],"|cffffd244http://www.wowace.com/addons/deus-vox-encounters|r"),
					order = 300,
				},
				blank3 = genblank(350),
				blank4 = genblank(351),
				blank5 = genblank(352),
				info_desc = {
					type = "description",
					name = format("%s: %s",L.options["DXE updated to work with"],"|cffffff78Warlords of Draenor|r"),
					order = 400,
				},
				blank6 = genblank(450),
				newauthor_desc = {
					type = "description",
					name = format("%s: %s",L.options["DXE is now maintained by"],"|cffffd244Mäev|r"),
					order = 451,
				},
				blank7 = genblank(452),
				blank8 = genblank(453),
				blank9 = genblank(454),	
				translators_desc = {
					type = "description",
					name = format("|cffffff78%s:",L.options["Translators"]),
					order = 500,
				},
				blank10 = genblank(501),	
				translators1_desc = {
					type = "description",
					name = format("%s: %s","ruRU","|cffffd200horws|r, |cffffd200badma|r"),
					order = 502,
				},	
				translators2_desc = {
					type = "description",
					name = format("%s: %s","zhCN","|cffffd200nrg3331|r, |cffffd200ananhaid|r"),
					order = 503,
				},	
				translators3_desc = {
					type = "description",
					name = format("%s: %s","koKR","|cffffd200apozeeon|r"),
					order = 504,
				},
				translators4_desc = {
					type = "description",
					name = format("%s: %s","frFR","|cffffd200Pettigrow|r"),
					order = 505,
				},
				translators5_desc = {
					type = "description",
					name = format("%s: %s","deDE","|cffffd200Junkie|r, |cffffd200Fearendon|r"),
					order = 506,
				},	
				translators6_desc = {
					type = "description",
					name = format("%s: %s","esES","|cffffd200brb1234|r"),
					order = 507,
				},	
				translators7_desc = {
					type = "description",
					name = format("%s: %s","zhTW","|cffffd200ananhaid|r"),
					order = 507,
				},		
				translators8_desc = {
					type = "description",
					name = format("%s: %s","ptBR","|cffffd200lucasmucidas|r"),
					order = 508,
				},					
				blank11 = genblank(600),	
				blank12 = genblank(601),
				blank13 = genblank(602),
				specialtranslators_desc = {
					type = "description",
					name = format("%s",L.options["Special thanks to all current and previous translators, your work will always be appreciated"]),
					order = 603,
				},					
			},
		}
		opts_args.about_group = about_group
	end

	---------------------------------------------
	-- ENCOUNTERS
	---------------------------------------------

	do
		local loadselect
		local handler = {}
		local encs_group = {
			type = "group",
			name = L.options["Encounters"],
			order = 200,
			childGroups = "tab",
			handler = handler,
			args = {
				simple_mode = {
					type = "execute",
					name = L.options["Simple"],
					desc = L.options["This mode only allows you to enable or disable"],
					order = 1,
					func = "SimpleMode",
					width = "half",
				},
				advanced_mode = {
					type = "execute",
					name = L.options["Advanced"],
					desc = L.options["This mode has customization options"],
					order = 2,
					func = "AdvancedMode",
				},
				modules = {
					type = "select",
					name = L.options["Modules"],
					order = 3,
					get = function()
						loadselect = loadselect or next(addon.Loader.Z_MODS_LIST)
						return loadselect end,
					set = function(info,v) loadselect = v end,
					values = function() return addon.Loader.Z_MODS_LIST end,
					disabled = function() return not loadselect end,
				},
				load = {
					type = "execute",
					name = L.options["Load"],
					order = 4,
					func  = function() addon.Loader:Load(loadselect) end,
					disabled = function() return not loadselect end,
					--width = "half",
				},
			},
		}

		function handler:OnLoadZoneModule() loadselect = next(addon.Loader.Z_MODS_LIST) end
		addon.RegisterCallback(handler,"OnLoadZoneModule")

		opts_args.encs_group = encs_group
		local encs_args = encs_group.args

		-- SIMPLE MODE
		function handler:GetSimpleEnable(info) return db.profile.Encounters[info[#info-2]][info[#info]].enabled end
		function handler:SetSimpleEnable(info,v) db.profile.Encounters[info[#info-2]][info[#info]].enabled = v end

		-- Can only enable/disable outputs
		local function InjectSimpleOptions(data,enc_args)
			for optionType,optionInfo in pairs(addon.EncDefaults) do
				local encData = data[optionType]
				local override = optionInfo.override
				if encData or override then
					enc_args[optionType] = enc_args[optionType] or {
						type = "group",
						name = optionInfo.L,
						order = optionInfo.order,
						args = {},
					}
					enc_args[optionType].inline = true
					enc_args[optionType].childGroups = nil

					local option_args = enc_args[optionType].args

					for var,info in pairs(override and optionInfo.list or encData) do
						option_args[var] = option_args[var] or {
							name = info.varname,
							width = "full",
						}
						option_args[var].type = "toggle"
						option_args[var].args = nil
						option_args[var].set = "SetSimpleEnable"
						option_args[var].get = "GetSimpleEnable"
					end
				end
			end
		end

		-- ADVANCED MODE

		local AdvancedItems = {
			VersionHeader = {
				type = "header",
				name = function(info)
					local version = EDB[info[#info-1]].version or "|cff808080"..L.options["Unknown"].."|r"
					return L.options["Version"]..": |cff99ff33"..version.."|r"
				end,
				order = 1,
				width = "full",
			},
			EnabledToggle = {
				type = "toggle",
				name = L.options["Enabled"],
				--[[name = function(info)
					local key,var = info[3],info[5]
					--return "|T"..EDB[key].alerts[var].icon..":20:20:-5|t"..EDB[key].alerts[var].ability
					if EDB[key].alerts[var].ability then
						return L.options["Enabled"].."   |T"..EDB[key].alerts[var].icon..":20:20:-5|t "..EDB[key].alerts[var].ability--L.options["Ability info"]
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--return "|T"..EDB[key].alerts[var].icon..":20:20:-5|t"..EDB[key].alerts[var].ability
					if EDB[key].alerts[var].ability then
						return "|T"..EDB[key].alerts[var].icon..":20:20:-5|t"..EDB[key].alerts[var].ability
					end
				end,--]]
				width = "half",
				order = 2,										-- data.key     -- info.var
				set = function(info,v)
					db.profile.Encounters[info[#info-3]][info[#info-1]].enabled = v 
				end,
				get = function(info) 
					local key,var = info[3],info[5]
					--print("asdasd",info[1],info[2],info[3],info[4],info[5],info[6])
					--print("Ddddd",info[4])
					local defn = EDB[key].alerts[var] or ""
					if info[4] == "messages" then
						--print("Ddddd2",info[4])
						defn = EDB[key].messages[var] or ""
					elseif info[4] == "arrows" then
						defn = EDB[key].arrows[var] or ""	
					elseif info[4] == "announces" then
						defn = EDB[key].announces[var] or ""							
					end
					local stgs = db.profile.Encounters[info[#info-3]][info[#info-1]]
					if defn ~= "" then
						--print("asdasd",EDB[key].alerts[var].enabled,stgs.encounterenabled,defn.enabled)
						if stgs.encounterenabled == nil and defn.enabled == false then
							stgs.enabled = false
							stgs.encounterenabled = false
						end
					end
					return db.profile.Encounters[info[#info-3]][info[#info-1]].enabled 
				end,
			},
			Header1 = {
				type = "execute",
				--name = L.options["Info"],
				name = function(info)
					local key,var = info[3],info[5]
					if EDB[key].alerts[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].alerts[var].ability)
						if a then
							if EDB[key].alerts[var].icon then
								return "  |T"..EDB[key].alerts[var].icon..":18:18:-5|t "..a--L.options["Info"]
							else
								return a
							end
						else
							return L.options["Info"]
						end
					elseif EDB[key].alerts[var].desc then
						local a = EDB[key].alerts[var].desc
						if a then
							local test = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(a, GetSpellInfo(a))
							return "  |T"..select(3,GetSpellInfo(a))..":18:18:-5|t "..GetSpellInfo(a)
						end
					end
				end,
				order = 3,
				width = "double",
				func = function(info)
					local key,var = info[3],info[5]
					--local alert = EDB[key].alerts[var].ability or ""
					if EDB[key].alerts[var].ability then
						local _, description,_,_,_,_,_,_, ability = EJ_GetSectionInfo(EDB[key].alerts[var].ability)
						--local description = select(2, EJ_GetSectionInfo(alert))
						--local ability = select(9, EJ_GetSectionInfo(alert))
						if IsInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif IsInGroup() then SendChatMessage(ability..": "..description, "PARTY")
						else addon:Print(ability..": "..description) end
					elseif EDB[key].alerts[var].desc then
						--print("asda",IsInRaid("player"))
						local description = GetSpellDescription(EDB[key].alerts[var].desc)
						local ability = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(EDB[key].alerts[var].desc, GetSpellInfo(EDB[key].alerts[var].desc))					
						if IsInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif IsInGroup() then SendChatMessage(ability..": "..description, "PARTY")
						else addon:Print(ability..": "..description) end
					end
				end,
				hidden = function(info)
					local key,var = info[3],info[5]
					if EDB[key].alerts[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].alerts[var].ability)
						return not a
					elseif EDB[key].alerts[var].desc then
						local a = GetSpellInfo(EDB[key].alerts[var].desc)
						return not a
					else
						return (not EDB[key].alerts[var].ability) or (not EDB[key].alerts[var].desc)
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--local alertsa = EDB[key].alerts[var].ability or ""
					if EDB[key].alerts[var].ability then
						local _, description = EJ_GetSectionInfo(EDB[key].alerts[var].ability)
						--print("A",title, description, abilityIcon, displayInfo)
						--return "  |T"..EDB[key].alerts[var].icon..":24:24:-5|t "..description
						return "\n"..description
						--return  "  |T"..EDB[key].alerts[var].icon..":18:18:-5|t "..a.." \n\n"..description
					elseif EDB[key].alerts[var].desc then
						return "\n"..GetSpellDescription(EDB[key].alerts[var].desc)
					end
				end,
			},
			Header2 = {
				type = "execute",
				--name = L.options["Info"],
				name = function(info)
					local key,var = info[3],info[5]
					if EDB[key].messages[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].messages[var].ability)
						if EDB[key].messages[var].icon then
							return "  |T"..EDB[key].messages[var].icon..":18:18:-5|t "..a--L.options["Info"]
						else
							return a
						end
					elseif EDB[key].messages[var].desc then
						local a = EDB[key].messages[var].desc
						if a then
							local test = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(a, GetSpellInfo(a))
							return "  |T"..select(3,GetSpellInfo(a))..":18:18:-5|t "..GetSpellInfo(a)
						end						
					end
				end,
				order = 4,
				width = "double",
				func = function(info)
					local key,var = info[3],info[5]
					--local alert = EDB[key].messages[var].ability or ""
					if EDB[key].messages[var].ability then
						local _, description,_,_,_,_,_,_, ability = EJ_GetSectionInfo(EDB[key].messages[var].ability)
						--local description = select(2, EJ_GetSectionInfo(alert))
						--local ability = select(9, EJ_GetSectionInfo(alert))
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					elseif EDB[key].messages[var].desc then
						local description = GetSpellDescription(EDB[key].messages[var].desc)
						local ability = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(EDB[key].messages[var].desc, GetSpellInfo(EDB[key].messages[var].desc))					
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					end
				end,
				hidden = function(info)
					local key,var = info[3],info[5]
					if EDB[key].messages[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].messages[var].ability)
						return not a
					elseif EDB[key].messages[var].desc then
						local a = GetSpellInfo(EDB[key].messages[var].desc)
						return not a
					else
						return (not EDB[key].messages[var].ability) or (not EDB[key].messages[var].desc)
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--local alertsa = EDB[key].messages[var].ability or ""
					if EDB[key].messages[var].ability then
						local _, description = EJ_GetSectionInfo(EDB[key].messages[var].ability)
						--print("A",title, description, abilityIcon, displayInfo)
						--return "  |T"..EDB[key].messages[var].icon..":24:24:-5|t "..description
						return "\n"..description
						--return  "  |T"..EDB[key].messages[var].icon..":18:18:-5|t "..a.." \n\n"..description
					elseif EDB[key].messages[var].desc then
						return "\n"..GetSpellDescription(EDB[key].messages[var].desc)
					end
				end,
			},
			Header3 = {
				type = "execute",
				--name = L.options["Info"],
				name = function(info)
					local key,var = info[3],info[5]
					if EDB[key].announces[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].announces[var].ability)
						if EDB[key].announces[var].icon then
							return "  |T"..EDB[key].announces[var].icon..":18:18:-5|t "..a--L.options["Info"]
						else
							return a
						end
					elseif EDB[key].announces[var].desc then
						local a = EDB[key].announces[var].desc
						if a then
							local test = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(a, GetSpellInfo(a))
							return "  |T"..select(3,GetSpellInfo(a))..":18:18:-5|t "..GetSpellInfo(a)
						end						
					end
				end,
				order = 4,
				width = "double",
				func = function(info)
					local key,var = info[3],info[5]
					--local alert = EDB[key].announces[var].ability or ""
					if EDB[key].announces[var].ability then
						local _, description,_,_,_,_,_,_, ability = EJ_GetSectionInfo(EDB[key].announces[var].ability)
						--local description = select(2, EJ_GetSectionInfo(alert))
						--local ability = select(9, EJ_GetSectionInfo(alert))
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					elseif EDB[key].announces[var].desc then
						local description = GetSpellDescription(EDB[key].announces[var].desc)
						local ability = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(EDB[key].announces[var].desc, GetSpellInfo(EDB[key].announces[var].desc))					
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					end
				end,
				hidden = function(info)
					local key,var = info[3],info[5]
					if EDB[key].announces[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].announces[var].ability)
						return not a
					elseif EDB[key].announces[var].desc then
						local a = GetSpellInfo(EDB[key].announces[var].desc)
						return not a
					else
						return (not EDB[key].announces[var].ability) or (not EDB[key].announces[var].desc)
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--local alertsa = EDB[key].announces[var].ability or ""
					if EDB[key].announces[var].ability then
						local _, description = EJ_GetSectionInfo(EDB[key].announces[var].ability)
						--print("A",title, description, abilityIcon, displayInfo)
						--return "  |T"..EDB[key].announces[var].icon..":24:24:-5|t "..description
						return "\n"..description
						--return  "  |T"..EDB[key].announces[var].icon..":18:18:-5|t "..a.." \n\n"..description
					elseif EDB[key].announces[var].desc then
						return "\n"..GetSpellDescription(EDB[key].announces[var].desc)
					end
				end,
			},
			Header4 = {
				type = "execute",
				--name = L.options["Info"],
				name = function(info)
					local key,var = info[3],info[5]
					if EDB[key].arrows[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].arrows[var].ability)
						if EDB[key].arrows[var].icon then
							return "  |T"..EDB[key].arrows[var].icon..":18:18:-5|t "..a--L.options["Info"]
						else
							return a
						end
					elseif EDB[key].arrows[var].desc then
						local a = EDB[key].arrows[var].desc
						if a then
							local test = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(a, GetSpellInfo(a))
							return "  |T"..select(3,GetSpellInfo(a))..":18:18:-5|t "..GetSpellInfo(a)
						end						
					end						
				end,
				order = 4,
				width = "double",
				func = function(info)
					local key,var = info[3],info[5]
					--local alert = EDB[key].arrows[var].ability or ""
					if EDB[key].arrows[var].ability then
						local _, description,_,_,_,_,_,_, ability = EJ_GetSectionInfo(EDB[key].arrows[var].ability)
						--local description = select(2, EJ_GetSectionInfo(alert))
						--local ability = select(9, EJ_GetSectionInfo(alert))
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					elseif EDB[key].arrows[var].desc then
						local description = GetSpellDescription(EDB[key].arrows[var].desc)
						local ability = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(EDB[key].arrows[var].desc, GetSpellInfo(EDB[key].arrows[var].desc))					
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					end
				end,
				hidden = function(info)
					local key,var = info[3],info[5]
					if EDB[key].arrows[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].arrows[var].ability)
						return not a
					elseif EDB[key].arrows[var].desc then
						local a = GetSpellInfo(EDB[key].arrows[var].desc)
						return not a
					else
						return (not EDB[key].arrows[var].ability) or (not EDB[key].arrows[var].desc)
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--local alertsa = EDB[key].arrows[var].ability or ""
					if EDB[key].arrows[var].ability then
						local _, description = EJ_GetSectionInfo(EDB[key].arrows[var].ability)
						--print("A",title, description, abilityIcon, displayInfo)
						--return "  |T"..EDB[key].arrows[var].icon..":24:24:-5|t "..description
						return "\n"..description
						--return  "  |T"..EDB[key].arrows[var].icon..":18:18:-5|t "..a.." \n\n"..description
					elseif EDB[key].arrows[var].desc then
						return "\n"..GetSpellDescription(EDB[key].arrows[var].desc)
					end
				end,
			},
			Header5 = {
				type = "execute",
				--name = L.options["Info"],
				name = function(info)
					local key,var = info[3],info[5]
					if EDB[key].raidicons[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].raidicons[var].ability)
						if EDB[key].raidicons[var].icon2 then
							return "  |T"..EDB[key].raidicons[var].icon2..":18:18:-5|t "..a--L.options["Info"]
						else
							return a
						end
					elseif EDB[key].raidicons[var].desc then
						local a = EDB[key].raidicons[var].desc
						if a then
							local test = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(a, GetSpellInfo(a))
							return "  |T"..select(3,GetSpellInfo(a))..":18:18:-5|t "..GetSpellInfo(a)
						end						
					end						
				end,
				order = 4,
				width = "double",
				func = function(info)
					local key,var = info[3],info[5]
					--local alert = EDB[key].raidicons[var].ability or ""
					if EDB[key].raidicons[var].ability then
						local _, description,_,_,_,_,_,_, ability = EJ_GetSectionInfo(EDB[key].raidicons[var].ability)
						--local description = select(2, EJ_GetSectionInfo(alert))
						--local ability = select(9, EJ_GetSectionInfo(alert))
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					elseif EDB[key].raidicons[var].desc then
						local description = GetSpellDescription(EDB[key].raidicons[var].desc)
						local ability = ("|cff71d5ff|Hspell:%d|h%s|h|r"):format(EDB[key].raidicons[var].desc, GetSpellInfo(EDB[key].raidicons[var].desc))					
						if UnitInRaid("player") then SendChatMessage(ability..": "..description, "RAID")
						elseif GetNumSubgroupMembers() > 0 then SendChatMessage(ability..": "..description, "PARTY")
						else print(ability..": "..description) end
					end
				end,
				hidden = function(info)
					local key,var = info[3],info[5]
					if EDB[key].raidicons[var].ability then
						local a, _ = EJ_GetSectionInfo(EDB[key].raidicons[var].ability)
						return not a
					elseif EDB[key].raidicons[var].desc then
						local a = GetSpellInfo(EDB[key].raidicons[var].desc)
						return not a
					else
						return (not EDB[key].raidicons[var].ability) or (not EDB[key].raidicons[var].desc)
					end
				end,
				desc = function(info)
					local key,var = info[3],info[5]
					--local alertsa = EDB[key].raidicons[var].ability or ""
					if EDB[key].raidicons[var].ability then
						local _, description = EJ_GetSectionInfo(EDB[key].raidicons[var].ability)
						--print("A",title, description, abilityIcon, displayInfo)
						--return "  |T"..EDB[key].raidicons[var].icon..":24:24:-5|t "..description
						return "\n"..description
						--return  "  |T"..EDB[key].raidicons[var].icon..":18:18:-5|t "..a.." \n\n"..description
					elseif EDB[key].raidicons[var].desc then
						return "\n"..GetSpellDescription(EDB[key].raidicons[var].desc)
					end
				end,
			},
			Options = {
				alerts = {
				--[[	header = {
						type = "header",
						name = function(info)
							local key,var = info[3],info[5]
							return EDB[key].alerts[var].varname
						end,
						order = 90,
						width = "full",
					},--]]
					--[[header = {
						type = "description",
						name = function(info)
							local key,var = info[3],info[5]
							--return "|T"..EDB[key].alerts[var].icon..":20:20:-5|t"..EDB[key].alerts[var].ability
							if EDB[key].alerts[var].ability then
								return EDB[key].alerts[var].ability
							end
						end,
						order = 90,
						width = "full",
					},--]]
					color1 = {
						type = "select",
						name = L.options["Main Color"],
						order = 100,
						values = "GetColor1",
					},
					color2 = {
						type = "select",
						name = L.options["Flash Color"],
						order = 200,
						values = "GetColor2",
						disabled = function(info)
							local key,var = info[3],info[5]
							return (not db.profile.Encounters[key][var].enabled) or (EDB[key].alerts[var].type == "simple") or (EDB[key].alerts[var].type == "inform")
						end,
					},
					sound = {
						type = "select",
						name = L.options["Sound"],
						order = 300,
						values = GetSounds,
					},
					--blank = genblank(350),
					counter = {
						type = "toggle",
						name = L.options["Counter"],
						order = 500,
						width = "half",
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((EDB[key].alerts[var].type == "debuff") or (EDB[key].alerts[var].counter))
						end,						
					},
					biggerbar = {
						type = "toggle",
						name = L.options["Bigger Bar"],
						desc = L.options["This option makes the selected alert bar to be bigger than the others, its great for focusing a especified bar (ex: aoe ability cooldown)"],
						order = 350,
						--set = function(info,v) EDB[key].alerts[var].biggerbar = v end,
						--get = function(info) return EDB[key].alerts[var].biggerbar end,
						disabled = function(info)
							local key,var = info[3],info[5]
							return not ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "debuff") or (EDB[key].alerts[var].type == "centerpopup"))
						end,
					},
					flashscreen = {
						type = "toggle",
						name = L.options["Flash Screen"],
						order = 400,
					},
					audiocd = {
						type = "toggle",
						name = L.options["Audio Countdown"],
						order = 500,
						disabled = function(info)
							local key,var = info[3],info[5]
							return not ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "centerpopup"))
						end,
					},
					flashtime = {
						type = "range",
						name = L.options["Flashtime"],
						desc = L.options["Flashtime in seconds left on alert"],
						order = 550,
						min = 1,
						max = 20,
						step = 1,
						disabled = function(info)
							local key,var = info[3],info[5]
							return not ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "centerpopup"))
						end,
						get = function(info)
							local key,var = info[3],info[5]
							if not db.profile.Encounters[key][var].flashtime then
								if EDB[key].alerts[var].flashtime then return EDB[key].alerts[var].flashtime
								else return db.profile.Globals.TransitionTime end
							else
								return db.profile.Encounters[key][var].flashtime
							end
						end,
					},
					audiotime = {
						type = "range",
						name = L.options["Audio Time"],
						desc = L.options["Audio seconds voice counter"],
						order = 560,
						min = 3,
						max = 10,
						step = 1,
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((db.profile.Encounters[key][var].audiocd == false))
							--return ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "centerpopup"))
						end,
					},
					type = {
						type = "select",
						name = L.options["Type"],
						order = 561,
						values = function(info)
							WarningType = {
								["simple"] = L.options["Warning"],
								["inform"] = L.options["Inform"],
								["message"] = L.options["Message"],
							}
							WarningType2 = {
								["dropdown"] = L.options["Top"],
								["centerpopup"] = L.options["Center"],
							}
							WarningType3 = {
								["debuff"] = L.options["Debuff"],
								["centerpopup"] = L.options["Center"],
							}
							WarningType4 = {
								["absorb"] = L.options["Absorb"],
							}
							local key,var = info[3],info[5]
							if EDB[key].alerts[var].type == "simple" or EDB[key].alerts[var].type == "inform" then
								return WarningType
							elseif EDB[key].alerts[var].type == "debuff" then
								return WarningType3
							elseif EDB[key].alerts[var].type == "absorb" then
								return WarningType4
							else return WarningType2
							end
						end,
						get = function(info)
							local key,var = info[3],info[5]
							--print("teste",db.profile.Encounters[key][var].type,EDB[key].alerts[var].type)
							if db.profile.Encounters[key][var].type then return db.profile.Encounters[key][var].type
							else return EDB[key].alerts[var].type end
						end,
						disabled = function(info)
							local key,var = info[3],info[5]
							return (EDB[key].alerts[var].type == "absorb")
						end,
					},
					textsize = {
						type = "range",
						name = L.options["Text Size"],
						desc = L.options["Custom Text Size for this ability"],
						order = 562,
						min = 10,
						max = 35,
						step = 1,
						get = function(info)
							local key,var = info[3],info[5]
							--print("teste",db.profile.Encounters[key][var].textsize,EDB[key].alerts[var].textsize)
							if not ((EDB[key].alerts[var].type == "simple") or (EDB[key].alerts[var].type == "inform")) then return 0 end
							if db.profile.Encounters[key][var].textsize == 0 then
								if db.profile.Encounters[key][var].type == "simple" then return 35
								else return 26 end
							else
								return db.profile.Encounters[key][var].textsize
							end
						end,
						set = function(info,v)
							local key,var = info[3],info[5]
						--	print("asd",v,db.profile.Encounters[key][var].textsize)
							db.profile.Encounters[key][var].textsize = v
						end,
						disabled = function(info)
							local key,var = info[3],info[5]
							return not ((db.profile.Encounters[key][var].type == "simple") or (db.profile.Encounters[key][var].type == "inform"))
						end,
					},
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 600,
						func = "TestAlert",
						disabled = function(info)
							local key,var = info[#info-4],info[#info-2]
							--local info = EDB[key].alerts[var]
							return (not db.profile.Encounters[key][var].enabled) or (EDB[key].alerts[var].type == "absorb")
							--return info.type == "absorb"
						end,
					},
					reset = {
						type = "execute",
						name = L.options["Reset"],
						order = 700,
						func = function(info)
							local key,var = info[3],info[5]
							local defaults = addon.defaults.profile.Encounters[key][var]
							local vardb = db.profile.Encounters[key][var]
								
							for k,v in pairs(defaults) do vardb[k] = v end
							if EDB[key].alerts[var].enabled == false then
								vardb["enabled"] = false
							elseif not EDB[key].alerts[var].flashtime then
								vardb["flashtime"] = db.profile.Globals.TransitionTime
							elseif EDB[key].alerts[var].flashtime then
								vardb["flashtime"] = EDB[key].alerts[var].flashtime						
							end	

							if EDB[key].alerts[var].type == "simple" then vardb.textsize = db.profile.TopMessageAnchor.TopMessageSize end
							if EDB[key].alerts[var].type == "inform" then vardb.textsize = db.profile.InformMessageAnchor.InformMessageSize end
						end,
					},
					headerrole = {
						type = "header",
						name = L.options["Exclude"],
						order = 701,
					},
					exhealer = {
						type = "toggle",
						name = L.options["Healers"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					extank = {
						type = "toggle",
						name = L.options["Tanks"],
						--desc = "teste",
						width = "half",
						order = 703,
					},
					exdps = {
						type = "toggle",
						name = L.options["Dps"],
						--desc = "teste",
						width = "half",
						order = 704,
					},
					exmelee = {
						type = "toggle",
						name = L.options["Melees"],
						--desc = "teste",
						width = "half",
						order = 705,
					},
					exranged = {
						type = "toggle",
						name = L.options["Ranged"],
						--desc = "teste",
						width = "half",
						order = 706,
					},
					excaster = {
						type = "toggle",
						name = L.options["Casters"],
						--desc = "teste",
						width = "half",
						order = 707,
					},				
					exlfr = {
						type = "toggle",
						name = L.options["LFR"],
						--desc = "teste",
						width = "half",
						order = 708,
					},
					exflex = {
						type = "toggle",
						name = L.options["Normal"],
						--desc = "teste",
						width = "half",
						order = 709,
					},
					exheroic = {
						type = "toggle",
						name = L.options["Heroic"],
						--desc = "teste",
						width = "half",
						order = 710,
					},
					exmythic = {
						type = "toggle",
						name = L.options["Mythic"],
						--desc = "teste",
						width = "half",
						order = 711,
					},					
					--[[maxplayers = {
						type = "toggle",
						name = L.options["Raid Size"],
						order = 709,
						width = "half",
					},					
					maxplayerssize = {
						type = "range",
						name = L.options["Max Players"],
						desc = L.options["Custom Max Players Size for this ability to show"],
						order = 710,
						min = 10,
						max = 40,
						step = 1,
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((db.profile.Encounters[key][var].maxplayers == false))
							--return ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "centerpopup"))
						end,						
					},			--]]
		
					--[[desc = {
						type = "description",
						name = L.options["Teste"],
						order = 702,
						width = "full",
					},--]]
				},
				raidicons = {
					desc = {
						type = "description",
						order = 100,
						name = function(info)
							local key,var = info[3],info[5]
							local varData = EDB[key].raidicons[var]
							local type = varData.type
							if type == "FRIENDLY" or type == "ENEMY" then
								return format(L.options["Uses |cffffd200Icon %s|r"],varData.icon)
							elseif type == "MULTIFRIENDLY" or type == "MULTIENEMY" then
								return format(L.options["Uses |cffffd200Icon %s|r to |cffffd200Icon %s|r"],varData.icon,varData.icon + varData.total - 1)
							end
						end,
					},
					headerrole = {
						type = "header",
						name = L.options["Exclude"],
						order = 701,
					},	
					exlfr = {
						type = "toggle",
						name = L.options["LFR"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					exflex = {
						type = "toggle",
						name = L.options["Normal"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					exheroic = {
						type = "toggle",
						name = L.options["Heroic"],
						--desc = "teste",
						width = "half",
						order = 703,
					},
					exmythic = {
						type = "toggle",
						name = L.options["Mythic"],
						--desc = "teste",
						width = "half",
						order = 704,
					},
				},
				arrows = {
					sound = {
						type = "select",
						name = L.options["Sound"],
						order = 100,
						values = GetSounds,
					},
					headerrole = {
						type = "header",
						name = L.options["Exclude"],
						order = 701,
					},
					exhealer = {
						type = "toggle",
						name = L.options["Healers"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					extank = {
						type = "toggle",
						name = L.options["Tanks"],
						--desc = "teste",
						width = "half",
						order = 703,
					},
					exdps = {
						type = "toggle",
						name = L.options["Dps"],
						--desc = "teste",
						width = "half",
						order = 704,
					},
					exmelee = {
						type = "toggle",
						name = L.options["Melees"],
						--desc = "teste",
						width = "half",
						order = 705,
					},
					exranged = {
						type = "toggle",
						name = L.options["Ranged"],
						--desc = "teste",
						width = "half",
						order = 706,
					},
					excaster = {
						type = "toggle",
						name = L.options["Casters"],
						--desc = "teste",
						width = "half",
						order = 707,
					},				
					exlfr = {
						type = "toggle",
						name = L.options["LFR"],
						--desc = "teste",
						width = "half",
						order = 708,
					},	
					exflex = {
						type = "toggle",
						name = L.options["Normal"],
						--desc = "teste",
						width = "half",
						order = 709,
					},
					exheroic = {
						type = "toggle",
						name = L.options["Heroic"],
						--desc = "teste",
						width = "half",
						order = 710,
					},
					exmythic = {
						type = "toggle",
						name = L.options["Mythic"],
						--desc = "teste",
						width = "half",
						order = 711,
					},					
				},
				announces = {
			--		sayyell = {
			--			type = "toggle",
			--			name = L.options["Replace Me with "],
			--			order = 200,
			--		},
					type = {
						type = "select",
						name = L.options["Type"],
						order = 100,
						values = function(info)
							AnnounceType = {
								["SAY"] = L.options["SAY"],
								["YELL"] = L.options["YELL"],
							}
							local key,var = info[3],info[5]
							if EDB[key].announces[var].type == "SAY" or EDB[key].announces[var].type == "YELL" then
								return AnnounceType
							--[[elseif EDB[key].alerts[var].type == "debuff" then
								return WarningType3
							elseif EDB[key].alerts[var].type == "absorb" then
								return WarningType4
							else return WarningType2--]]
							end
						end,
						get = function(info)
							local key,var = info[3],info[5]
							--print("teste",db.profile.Encounters[key][var].type,EDB[key].alerts[var].type)
							if db.profile.Encounters[key][var].type then return db.profile.Encounters[key][var].type
							else return EDB[key].announces[var].type end
						end,
						--width = "half",
						--disabled = function(info)
						--	local key,var = info[3],info[5]
						--	return (EDB[key].alerts[var].type == "debuff") or (EDB[key].alerts[var].type == "absorb")
						--end,
					},
					--[[Throttle = {
						order = 104,
						type = "range",
						name = L.options["Delay"],
						desc = L.options["Adjust the delay of sound and flash screen alerts notifies you again"],
						min = 1,
						max = 15,
						step = 1,
					},
					ReplaceMe = {
						type = "toggle",
						name = function(info)
							local class = select(2,UnitClass("player"))
							local name = UnitName("player")
							return L.options["Replace Me with "]..class_color[class]..name
						end,
						width = "double",
						order = 105,
					},	--]]
					blank = genblank(199),
					flashcolor = {
						type = "select",
						name = L.options["Flash Color"],
						order = 200,
						values = "GetColor2",
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((db.profile.Encounters[key][var].enabled ~= true))
						end,
					},
					sound = {
						type = "select",
						name = L.options["Sound"],
						order = 201,
						values = GetSounds,
					},
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 202,
						func = "TestAnnounce",
						disabled = function(info)
							local key,var = info[#info-4],info[#info-2]
							--local info = EDB[key].alerts[var]
							return ((db.profile.Encounters[key][var].enabled ~= true))
							--return info.type == "absorb"
						end,
					},		
					reset = {
						type = "execute",
						name = L.options["Reset"],
						order = 203,
						func = function(info)
							local key,var = info[3],info[5]
							local defaults = addon.defaults.profile.Encounters[key][var]
							local vardb = db.profile.Encounters[key][var]
							for k,v in pairs(defaults) do vardb[k] = v end
							if EDB[key].announces[var].enabled == false then
								vardb["enabled"] = false
							end
						end,
					},					
					headerrole = {
						type = "header",
						name = L.options["Exclude"],
						order = 701,
					},
					exhealer = {
						type = "toggle",
						name = L.options["Healers"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					extank = {
						type = "toggle",
						name = L.options["Tanks"],
						--desc = "teste",
						width = "half",
						order = 703,
					},
					exdps = {
						type = "toggle",
						name = L.options["Dps"],
						--desc = "teste",
						width = "half",
						order = 704,
					},
					exmelee = {
						type = "toggle",
						name = L.options["Melees"],
						--desc = "teste",
						width = "half",
						order = 705,
					},
					exranged = {
						type = "toggle",
						name = L.options["Ranged"],
						--desc = "teste",
						width = "half",
						order = 706,
					},
					excaster = {
						type = "toggle",
						name = L.options["Casters"],
						--desc = "teste",
						width = "half",
						order = 707,
					},				
					exlfr = {
						type = "toggle",
						name = L.options["LFR"],
						--desc = "teste",
						width = "half",
						order = 708,
					},
					exflex = {
						type = "toggle",
						name = L.options["Normal"],
						--desc = "teste",
						width = "half",
						order = 709,
					},
					exheroic = {
						type = "toggle",
						name = L.options["Heroic"],
						--desc = "teste",
						width = "half",
						order = 710,
					},
					exmythic = {
						type = "toggle",
						name = L.options["Mythic"],
						--desc = "teste",
						width = "half",
						order = 711,
					},
					--[[player = {
						type = "toggle",
						name = function(info)
							--L.options["Replace Me with "]..UnitName("player"),
							local class = select(2,UnitClass("player"))
							local name = UnitName("player")
							--num = L.options["on"].." "..class_color[class]..name.."|r"
							return L.options["Replace Me with "]..class_color[class]..name
						end,
						--width = "full",
						order = 200,
					},--]]
				},
				messages = {
					--[[header = {
						type = "header",
						name = function(info)
							local key,var = info[3],info[5]
							return EDB[key].messages[var].varname
						end,
						order = 91,
						width = "full",
					},--]]
					--[[header = {
						type = "description",
						name = function(info)
							local key,var = info[3],info[5]
							--return "|T"..EDB[key].alerts[var].icon..":20:20:-5|t"..EDB[key].alerts[var].ability
							if EDB[key].messages[var].ability then
								return "  |T"..EDB[key].messages[var].icon..":28:28:-5|t"..EDB[key].messages[var].ability
							end
						end,
						order = 91,
						width = "full",
					},--]]
					color1 = {
						type = "select",
						name = L.options["Main Color"],
						order = 102,
						values = "GetColor1M",
					},
					sound = {
						type = "select",
						name = L.options["Sound"],
						order = 302,
						values = GetSounds,
					},
					type = {
						type = "select",
						name = L.options["Type"],
						order = 561,
						values = function(info)
							WarningType = {
								["simple"] = L.options["Warning"],
								["inform"] = L.options["Inform"],
								["message"] = L.options["Message"],
							}
							local key,var = info[3],info[5]
							--if EDB[key].alerts[var].type == "message" then
								return WarningType
							--end
						end,
						get = function(info)
							local key,var = info[3],info[5]
							--print("teste",db.profile.Encounters[key][var].type,EDB[key].alerts[var].type)
							if db.profile.Encounters[key][var].type then return db.profile.Encounters[key][var].type
							else return EDB[key].alerts[var].type end
						end,
						--disabled = function(info)
						--	local key,var = info[3],info[5]
						--	return (EDB[key].alerts[var].type == "absorb")
						--end,
					},
					textsize = {
						type = "range",
						name = L.options["Text Size"],
						desc = L.options["Custom Text Size for this ability"],
						order = 562,
						min = 10,
						max = 35,
						step = 1,
						get = function(info)
							local key,var = info[3],info[5]
							if not ((db.profile.Encounters[key][var].type == "simple") or (db.profile.Encounters[key][var].type == "inform")) then return 0 end					
							if not db.profile.Encounters[key][var].textsize then
								if db.profile.Encounters[key][var].type == "simple" then return 35
								else return 26 end
							else
								return db.profile.Encounters[key][var].textsize or 0
							end
						end,
						set = function(info,v)
							local key,var = info[3],info[5]					
							db.profile.Encounters[key][var].textsize = v
						end,
						disabled = function(info)
							local key,var = info[3],info[5]
							return not ((db.profile.Encounters[key][var].type == "simple") or (db.profile.Encounters[key][var].type == "inform"))
						end,
					},
					counter2 = {
						type = "toggle",
						name = L.options["Counter"],
						order = 590,
						width = "half",					
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((EDB[key].messages[var].counter2))
						end,						
					},
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 602,
						func = "TestMessage",
						disabled = function(info)
							local key,var = info[#info-4],info[#info-2]
							local info = EDB[key].messages[var]
							return not db.profile.Encounters[key][var].enabled
							--return info.type == "absorb"
						end,
					},
					reset = {
						type = "execute",
						name = L.options["Reset"],
						order = 603,
						func = function(info)
							local key,var = info[3],info[5]
							local defaults = addon.defaults.profile.Encounters[key][var]
							local vardb = db.profile.Encounters[key][var]
							for k,v in pairs(defaults) do vardb[k] = v end
							if EDB[key].messages[var].enabled == false then
								vardb["enabled"] = false
							end
						end,
					},
					headerrole = {
						type = "header",
						name = L.options["Exclude"],
						order = 701,
					},
					exhealer = {
						type = "toggle",
						name = L.options["Healers"],
						--desc = "teste",
						width = "half",
						order = 702,
					},
					extank = {
						type = "toggle",
						name = L.options["Tanks"],
						--desc = "teste",
						width = "half",
						order = 703,
					},
					exdps = {
						type = "toggle",
						name = L.options["Dps"],
						--desc = "teste",
						width = "half",
						order = 704,
					},
					exmelee = {
						type = "toggle",
						name = L.options["Melees"],
						--desc = "teste",
						width = "half",
						order = 705,
					},
					exranged = {
						type = "toggle",
						name = L.options["Ranged"],
						--desc = "teste",
						width = "half",
						order = 706,
					},
					excaster = {
						type = "toggle",
						name = L.options["Casters"],
						--desc = "teste",
						width = "half",
						order = 707,
					},
					exlfr = {
						type = "toggle",
						name = L.options["LFR"],
						--desc = "teste",
						width = "half",
						order = 708,
					},
					exflex = {
						type = "toggle",
						name = L.options["Normal"],
						--desc = "teste",
						width = "half",
						order = 709,
					},
					exheroic = {
						type = "toggle",
						name = L.options["Heroic"],
						--desc = "teste",
						width = "half",
						order = 710,
					},
					exmythic = {
						type = "toggle",
						name = L.options["Mythic"],
						--desc = "teste",
						width = "half",
						order = 711,
					},
					--[[maxplayers = {
						type = "toggle",
						name = L.options["Raid Size"],
						order = 709,
						width = "half",
					},					
					maxplayerssize = {
						type = "range",
						name = L.options["Max Players"],
						desc = L.options["Custom Max Players Size for this ability to show"],
						order = 710,
						min = 10,
						max = 40,
						step = 1,
						disabled = function(info)
							local key,var = info[3],info[5]
							return ((db.profile.Encounters[key][var].maxplayers == false))
							--return ((EDB[key].alerts[var].type == "dropdown") or (EDB[key].alerts[var].type == "centerpopup"))
						end,						
					},--]]					
				},
				windows = {
					-- prefix options with (%w+)window
					proxoverride = {
						type = "toggle",
						name = L.options["Custom range"],
						order = 100,
					},
					proxrange = {
						type = "range",
						name = L.options["Range"],
						order = 200,
						min = 2,
						max = 18,
						step = 1,
						disabled = function(info)
							local key = info[3]
							return not (db.profile.Encounters[key]["proxwindow"].enabled and db.profile.Encounters[key]["proxwindow"].proxoverride)
						end,
					},
					apboverride = {
						type = "toggle",
						name = L.options["Custom threshold"],
						order = 100,
					},
					apbthreshold = {
						type = "range",
						name = L.options["Threshold"],
						order = 200,
						min = 1,
						max = 99,
						step = 1,
						disabled = function(info)
							local key = info[3]
							return not (db.profile.Encounters[key]["apbwindow"].enabled and db.profile.Encounters[key]["apbwindow"].apboverride)
						end,
					},
				},
				--[[abilities = {
					desc = {
						type = "description",
						order = 1000,
						name = "Teste",
					},
				},--]]
			}
		}

		do
			local info_n = 7						 -- var
			local info_n_MINUS_4 = info_n - 4 -- type
			local info_n_MINUS_2 = info_n - 2 -- key

			local colors = {}
			local colors1simple = {}
			local colors2 = {}
			local colors3 = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
				colors1simple[k] = hex
				colors2[k] = hex
				colors3[k] = hex
			end
			colors1simple["Clear"] = L.options["Clear"]
			colors2["Off"] = OFF
			colors3["GLOBAL"] = L.options["GLOBAL"] --db.profile.Globals.GlobalBarCrl

			function handler:GetColor1(info)
				local key,var = info[3],info[5]

				if EDB[key].alerts[var].type == "simple" and not addon.Alerts.db.profile.EnableWarningMessage then
					return colors1simple
				elseif EDB[key].alerts[var].type == "dropdown" or EDB[key].alerts[var].type == "centerpopup" then
					return colors3
				else
					return colors
				end
			end
			function handler:GetColor1M(info)
				return colors
			end
			
			function handler:GetColor2()
				return colors2
			end
			
			function handler:DisableSettings(info)
				return not db.profile.Encounters[info[info_n_MINUS_4]][info[info_n_MINUS_2]].enabled
			end

			function handler:GetOption(info)
				return db.profile.Encounters[info[info_n_MINUS_4]][info[info_n_MINUS_2]][info[info_n]]
			end

			function handler:SetOption(info,v)
				db.profile.Encounters[info[info_n_MINUS_4]][info[info_n_MINUS_2]][info[info_n]] = v
			end

			local function replace_nums(num)
				if num == "11" then
					return "2"
				else
					local class = select(2,UnitClass("player"))
					local name = UnitName("player")
					--num = L.options["on"].." "..class_color[class]..name.."|r"
					num = class_color[class]..name.."|r"
					return num
				end
			end
			local function replace_vars(num)
				if num == "player" then
					local class = select(2,UnitClass("player"))
					local name = UnitName("player")
					--num = L.options["on"].." "..class_color[class]..name.."|r"
					num = class_color[class]..name.."|r"
					return num
				else
					return "2"
				end
			end
			function handler:TestAlert(info)
				local key,var = info[info_n_MINUS_4],info[info_n_MINUS_2]
				local info = EDB[key].alerts[var]
				local stgs = db.profile.Encounters[key][var]
				local str = gsub(info.text,"#(.-)#",replace_nums)
				local static = false
				str = gsub(str,"&(.-)&",replace_nums)
				str = gsub(str,"<(.-)>",replace_vars)
				if info.static then static = true end
				local flashtime = stgs.flashtime or pfl.Globals.TransitionTime or info.flashtime
				if stgs.counter and not info.counter then
					str = "(2) "..str
					--local c = counters[var] or 0
					--c = c + 1
					--str = "("..c..") "..str
					--counters[var] = c
				end

				if stgs.type == "dropdown" then
					addon.Alerts:Dropdown(var,str,flashtime+5,flashtime,stgs.sound,stgs.color1,stgs.color2,stgs.flashscreen,info.icon,stgs.audiocd,stgs.audiotime,stgs.biggerbar)
				elseif stgs.type == "centerpopup" then
					addon.Alerts:CenterPopup(var,str,10,flashtime,stgs.sound,stgs.color1,stgs.color2,stgs.flashscreen,info.icon,stgs.audiocd,stgs.audiotime,stgs.biggerbar,static)
				elseif stgs.type == "simple" then
					addon.Alerts:Simple(str,3,stgs.sound,stgs.color1,stgs.flashscreen,info.icon,stgs.textsize) --info.varname
				elseif stgs.type == "debuff" then
					addon.Alerts:DebuffPopup(var,str,flashtime+5,flashtime,stgs.sound,stgs.color1,stgs.color2,stgs.flashscreen,info.icon,stgs.audiocd,stgs.audiotime,stgs.biggerbar,static) --info.varname
				elseif stgs.type == "inform" then
					addon.Alerts:Inform(str,stgs.sound,stgs.color1,info.time,stgs.flashscreen,info.icon,stgs.textsize) -- info.varname
				elseif stgs.type == "message" then
					addon.Alerts:NewTMessage(str,stgs.color1,info.icon,stgs.sound)
				elseif stgs.type == "unitabsorb" then
					addon.Alerts:UnitAbsorb(var,str,info.textformat,info.time,info.flashtime,stgs.sound,stgs.color1,stgs.color2,stgs.flashscreen,info.icon,
							  info.spellid,UnitName("player"))	
				end
			end

			function handler:TestMessage(info)
				local key,var = info[info_n_MINUS_4],info[info_n_MINUS_2]
				local info = EDB[key].messages[var]
				local stgs = db.profile.Encounters[key][var]
				
				--local varname = info.varname
				local str = gsub(info.text,"#(.-)#",replace_nums)
				str = gsub(str,"&(.-)&",replace_nums)
				str = gsub(str,"<(.-)>",replace_vars)
				if stgs.counter2 and not info.counter2 then
					--local c = counters[var] or 0
					--c = c + 1
					str = str.." (2)"
					--counters[var] = c
				end
				
				if stgs.type == "message" then
					addon.Alerts:NewTMessage(str,stgs.color1,info.icon,stgs.sound)
				elseif stgs.type == "inform" then
					addon.Alerts:Inform(str,stgs.sound,stgs.color1,2,stgs.flashscreen,info.icon,stgs.textsize)					
				elseif stgs.type == "simple" then
					addon.Alerts:Simple(str,3,stgs.sound,stgs.color1,stgs.flashscreen,info.icon,stgs.textsize)					
				end
			end
			
			function handler:TestAnnounce(info)
				local key,var = info[info_n_MINUS_4],info[info_n_MINUS_2]
				local info = EDB[key].announces[var]
				local stgs = db.profile.Encounters[key][var]
				local omsg = info.msg
				omsg = gsub(info.msg,"#(.-)#",replace_nums)
				omsg = gsub(omsg,"&(.-)&",replace_nums)
				omsg = gsub(omsg,"<(.-)>",replace_vars)
				--if db.profile.Announces.ReplaceMe then omsg = omsg:gsub("(Me)", getclasscolor(UnitName("player"))) end
				--if db.profile.Announces.ReplaceMe then omsg = omsg:gsub("(Me)", UnitName("player")) end
				if stgs.ReplaceMe then
					local pStart, pEnd, capt = find(string.lower(omsg), "(%s+)" .. string.lower(L.alert["Me"]));
					if pStart then
						omsg = format("%s%s%s%s", sub(info.msg, 0, pStart - 1), capt, UnitName("player"), sub(info.msg, pEnd + 1));		
					end
				end				
				addon.Alerts:NewAnnounce(omsg,stgs.flashcolor,stgs.type,stgs.sound)
			end			
		end

		local function InjectAdvancedOptions(data,enc_args)
			-- Add output options
			for optionType,optionInfo in pairs(addon.EncDefaults) do
				local encData = data[optionType]
				local override = optionInfo.override
				if encData or override then
						enc_args[optionType] = enc_args[optionType] or {
							type = "group",
							name = optionInfo.L,
							order = optionInfo.order,
							args = {},
							
						}
						enc_args[optionType].inline = nil
						enc_args[optionType].childGroups = "select"

					local option_args = enc_args[optionType].args
					for var,info in pairs(override and optionInfo.list or encData) do
						
						option_args[var] = option_args[var] or {
							name = info.varname,
							width = "full",
							order = 1,
						}

						option_args[var].type = "group"
						option_args[var].args = {}
						option_args[var].get = nil
						option_args[var].set = nil

						local item_args = option_args[var].args
							item_args.enabled = AdvancedItems.EnabledToggle
													
							if AdvancedItems.Options[optionType] then
								item_args.settings = {
									type = "group",
									name = L.options["Settings"],
									order = 4,
									inline = true,
									disabled = "DisableSettings",
									get = "GetOption",
									set = "SetOption",
									args = {},
								}

								local settings_args = item_args.settings.args
								for k,item in pairs(AdvancedItems.Options[optionType]) do
									-- special handling for windows since not all windows share the same options
									if optionType == "windows" then
										-- prefix needs to match
										if k:find("^"..var:match("(%w+)window")) then
											settings_args[k] = item
										end
									elseif optionType == "alerts" then
										item_args.header1 = AdvancedItems.Header1
										settings_args[k] = item
									elseif optionType == "messages" then
										item_args.header1 = AdvancedItems.Header2
										settings_args[k] = item
									elseif optionType == "announces" then
										item_args.header1 = AdvancedItems.Header3
										settings_args[k] = item
									elseif optionType == "arrows" then
										item_args.header1 = AdvancedItems.Header4
										settings_args[k] = item
									elseif optionType == "raidicons" then
										item_args.header1 = AdvancedItems.Header5
										settings_args[k] = item
									else
										settings_args[k] = item
									end
								end
							end
						end
					--[[else
						enc_args[optionType] = enc_args[optionType] or {
							type = "group",
							name = optionInfo.L,
							order = optionInfo.order,
							args = {},
						}
						enc_args[optionType].inline = nil
						enc_args[optionType].childGroups = "select"
						
						local option_args = enc_args[optionType].args
						for var,info in pairs(override and optionInfo.list or encData) do
							option_args[var] = option_args[var] or {
								name = info.varname,
								width = "full",
							}

							option_args[var].type = "description"
							option_args[var].args = {}
							option_args[var].get = "GetOption"
							option_args[var].set = nil
						end
					end--]]
				end
			end
		end

		-- MODE SWAPPING

		local function SwapMode()
			for catkey in pairs(encs_args) do
				if encs_args[catkey].type == "group" then
					local cat_args = encs_args[catkey].args
					for key in pairs(cat_args) do
						local data = EDB[key]
						local enc_args = cat_args[key].args
						if db.global.AdvancedMode then
							InjectAdvancedOptions(data,enc_args)
						else
							InjectSimpleOptions(data,enc_args)
						end
					end
				end
			end
		end

		function handler:SimpleMode()
			if db.global.AdvancedMode then db.global.AdvancedMode = false; SwapMode() end
		end

		function handler:AdvancedMode()
			if not db.global.AdvancedMode then db.global.AdvancedMode = true; SwapMode() end
		end

		-- ADDITIONS/REMOVALS

		local function formatkey(str) return str:gsub(" ",""):lower() end

		local function AddEncounterOptions(data)
			-- Add a zone group if it doesn't exist. category supersedes zone
			local catkey = data.category and formatkey(data.category) or formatkey(data.zone)
			encs_args[catkey] = encs_args[catkey] or {type = "group", name = data.category or data.zone, args = {}}
			-- Update args pointer
			local cat_args = encs_args[catkey].args
			-- Exists, delete args
			if cat_args[data.key] then
				cat_args[data.key].args = {}
			else
				-- Add the encounter group
				cat_args[data.key] = {
					type = "group",
					name = data.name,
					desc = data.desc,
					childGroups = "tab",
					args = {
						version_header = AdvancedItems.VersionHeader,
					},
				}
			end
			-- Set pointer to the correct encounter group
			local enc_args = cat_args[data.key].args
			if db.global.AdvancedMode then
				InjectAdvancedOptions(data,enc_args)
			else
				InjectSimpleOptions(data,enc_args)
			end
		end


		local function RemoveEncounterOptions(data)
			local catkey = data.category and formatkey(data.category) or formatkey(data.zone)
			encs_args[catkey].args[data.key] = nil
			-- Remove category if there are no more encounters in it
			if not next(encs_args[catkey].args) then
				encs_args[catkey] = nil
			end
		end

		function handler:OnRegisterEncounter(event,data) AddEncounterOptions(data); ACR:NotifyChange("DXE") end
		function handler:OnUnregisterEncounter(event,data) RemoveEncounterOptions(data); ACR:NotifyChange("DXE") end

		addon.RegisterCallback(handler,"OnRegisterEncounter")
		addon.RegisterCallback(handler,"OnUnregisterEncounter")

		function module:FillEncounters() for key,data in addon:IterateEDB() do AddEncounterOptions(data) end end
	end

	---------------------------------------------
	-- ALERTS
	---------------------------------------------

	do
		local Alerts = addon.Alerts

		local function SetNoRefresh(info,v,v2,v3,v4)
			local var = info[#info]
			if var:find("Color") then
				local c = Alerts.db.profile[var]
				c[1],c[2],c[3],c[4] = v,v2,v3,v4
			else Alerts.db.profile[var] = v end
		end

		local SelectedEncounter
		local EncounterList = {}

		local alerts_group = {
			type = "group",
			name = L.options["Alerts"],
			order = 200,
			handler = Alerts,
			childGroups = "tab",
			get = function(info)
				local var = info[#info]
				if var:find("Color") then return unpack(Alerts.db.profile[var])
				else return Alerts.db.profile[var] end
			end,
			set = function(info,v,v2,v3,v4)
				local var = info[#info]
				if var:find("Color") then
					local c = Alerts.db.profile[var]
					c[1],c[2],c[3],c[4] = v,v2,v3,v4
				else Alerts.db.profile[var] = v end
				Alerts:RefreshBars()
			end,
			args = {}
		}

		opts_args.alerts_group = alerts_group
		local alerts_args = alerts_group.args

		local bars_group = {
			type = "group",
			name = L.options["Bars"],
			order = 100,
			args = {
			BarTest = {
					type = "execute",
					name = L.options["Test Bars"],
					desc = L.options["Fires a dropdown, center popup, simple, debuff and warning alert bars"],
					order = 100,
					func = "BarTest",
				},
				BarFillDirection = {
					order = 130,
					type = "select",
					name = L.options["Bar Fill Direction"],
					desc = L.options["The direction bars fill"],
					values = {
						FILL = L.options["Left to Right"],
						DEPLETE = L.options["Right to Left"],
					},
				},
				BarHeight = {
					order = 140,
					type = "range",
					name = L.options["Bar Height"],
					desc = L.options["Select a bar height"],
					min = 8,
					max = 40,
					step = 1,
				},
				BarSpacing = {
					order = 145,
					type = "range",
					name = L.options["Bar Spacing"],
					desc = L.options["How far apart the bars are spaced vertically"],
					min = 0,
					max = 300,
					step = 1,
				},
				ShowBarBorder = {
					order = 150,
					type = "toggle",
					name = L.options["Show Bar Border"],
					desc = L.options["Display a border around bars"],
					--width = "double",
				},
				DisableDropdowns = {
					order = 160,
					type = "toggle",
					name = L.options["Disable Dropdowns"],
					desc = L.options["Anchor bars onto the center anchor only"],
					set = SetNoRefresh,
				},
				EnableWarningMessage = {
					order = 170,
					type = "toggle",
					name = L.options["Enable Top Message"],
					desc = L.options["Enable Warning Message Instead of Warning Bars."],
					set = SetNoRefresh,
					--width = "half",
				},
				DebuffBars = {
					order = 180,
					type = "toggle",
					name = L.options["Enable Debuff Bar"],
					desc = L.options["Enable displaying Debuff stacks on Debuff Bar."],
					set = SetNoRefresh,
					--width = "half",
				},
			},
		}

		alerts_args.bars_group = bars_group
		local bars_args = bars_group.args

		local general_group = {
			type = "group",
			name = L.options["General"],
			order = 100,
			args = {

			},
		}

		local font_group = {
			type = "group",
			name = L.options["Text"],
			order = 400,
			args = {
				font_desc = {
					type = "header",
					name = L.options["Adjust the text used on timer bars"].."\n",
					order = 1,
				},
				bartext_group = {
					type = "group",
					name = L.options["Bar Text"],
					inline = true,
					order = 1,
					args = {
						TextXOffset = {
							order = 50,
							type = "range",
							name = L.options["Horizontal Offset"],
							desc = L.options["The horizontal position of the bar text"],
							min = -1000,
							max = 1000,
							step = 1,
						},
						TextYOffset = {
							order = 75,
							type = "range",
							name = L.options["Vertical Offset"],
							desc = L.options["The vertical position of the bar text"],
							min = -1000,
							max = 1000,
							step = 1,
						},
						BarFontSize = {
							order = 100,
							type = "range",
							name = L.options["Font Size"],
							desc = L.options["Select a font size used on bar text"],
							min = 8,
							max = 20,
							step = 1,
						},
						TextAlpha = {
							type = "range",
							name = L.options["Alpha"],
							desc = L.options["Adjust the transparency of bar text"],
							order = 150,
							min = 0.1,
							max = 1,
							step = 0.05,
						},
						BarTextJustification = {
							order = 170,
							type = "select",
							name = L.options["Justification"],
							desc = L.options["Select a text justification"],
							values = {
								LEFT = L.options["Left"],
								CENTER = L.options["Center"],
								RIGHT = L.options["Right"],
							},
						},
						BarFontColor = {
							order = 76,
							type = "color",
							name = L.options["Font Color"],
							desc = L.options["Set a font color used on bar text"],
							width = "half",
						},
						BarFontOutline = {
							order = 201,
							type = "select",
							name = L.options["Font Quality"],
							desc = L.options["The message font quality"],
							values = {DEFAULT = L.options["DEFAULT"], OUTLINE = L.options["OUTLINE"], THICKOUTLINE = L.options["THICKOUTLINE"]},			
						},
					},
				},
				timertext_group = {
					type = "group",
					name = L.options["Timer Text"],
					order = 2,
					inline = true,
					args = {
						timer_desc = {
							type = "description",
							name = L.options["Timer font sizes are determined by bar height"].."\n",
							order = 1,
						},
						TimerFontColor = {
							order = 53,
							type = "color",
							name = L.options["Font Color"],
							desc = L.options["Set a font color used on bar timers"],
							width = "half",
						},
						TimerFontOutlineLeft = {
							order = 51,
							type = "select",
							name = L.options["Seconds Font Quality"],
							desc = L.options["Select a font quality used on a timer's seconds text"],
							values = {OUTLINE = L.options["OUTLINE"], THICKOUTLINE = L.options["THICKOUTLINE"], DEFAULT = L.options["DEFAULT"]},			
						},	
						TimerFontOutlineRight = {
							order = 52,
							type = "select",
							name = L.options["Decimal Font Quality"],
							desc = L.options["Select a font quality used on a timer's decimal text"],
							values = {OUTLINE = L.options["OUTLINE"], DEFAULT = L.options["DEFAULT"]},			
						},	
						TimerXOffset = {
							order = 100,
							type = "range",
							name = L.options["Horizontal Offset"],
							desc = L.options["The horizontal position of the timer text"],
							min = -1000,
							max = 1000,
							step = 1,
						},
						TimerYOffset = {
							order = 200,
							type = "range",
							name = L.options["Vertical Offset"],
							desc = L.options["The vertical position of the timer text"],
							min = -1000,
							max = 1000,
							step = 1,
						},
						DecimalYOffset = {
							order = 300,
							type = "range",
							name = L.options["Decimal Vertical Offset"],
							desc = L.options["The vertical position of a timer's decimal text in relation to its second's text"],
							min = -10,
							max = 10,
							step = 1,
						},
						TimerAlpha = {
							type = "range",
							name = L.options["Alpha"],
							desc = L.options["Adjust the transparency of timer text"],
							order = 305,
							min = 0.1,
							max = 1,
							step = 0.05,
						},
						ScaleTimerWithBarHeight = {
							order = 307,
							type = "toggle",
							width = "normal",
							name = L.options["Scale with bar height"],
							desc = L.options["Automatically resizes timer font sizes when the bar height is adjusted"],
						},
						HideDecimalTimer = {
							order = 308,
							type = "toggle",
							width = "normal",
							name = L.options["Hide Decimal"],
						},						
						TimerSecondsFontSize = {
							order = 310,
							type = "range",
							name = L.options["Seconds' Font Size"],
							desc = L.options["Select a font size used on a timer's seconds' text"],
							min = 8,
							max = 30,
							step = 1,
							disabled = function() return Alerts.db.profile.ScaleTimerWithBarHeight end,
						},
						TimerDecimalFontSize = {
							order = 320,
							type = "range",
							name = L.options["Decimal Font Size"],
							desc = L.options["Select a font size used on a timer's decimal text"],
							min = 8,
							max = 16,
							step = 1,
							disabled = function() return Alerts.db.profile.ScaleTimerWithBarHeight end,
						},					
					},
				},
			},
		}

		bars_args.font_group = font_group

		local icon_group = {
			type = "group",
			name = L.options["Icon"],
			order = 500,
			args = {
				icon_desc = {
					type = "header",
					name = L.options["Adjust the spell icon on timer bars"].."\n",
					order = 1,
				},
				ShowLeftIcon = {
					order = 100,
					type = "toggle",
					name = L.options["Show Left Icon"],
					desc = L.options["Shows a spell icon on the left side of the bar"],
				},
				ShowRightIcon = {
					order = 125,
					type = "toggle",
					name = L.options["Show Right Icon"],
					desc = L.options["Shows a spell icon on the right side of the bar"],
				},
				ShowIconBorder = {
					order = 150,
					type = "toggle",
					name = L.options["Show Icon Border"],
					desc = L.options["Display a border around icons"],
				},
				blank = genblank(200),
				IconXOffset = {
					order = 300,
					type = "range",
					name = L.options["Icon X Offset"],
					desc = L.options["Horizontal position of the icon"],
					min = -200,
					max = 200,
					step = 0.1,
				},
				IconYOffset = {
					order = 400,
					type = "range",
					name = L.options["Icon Y Offset"],
					desc = L.options["Vertical position of the icon"],
					min = -200,
					max = 200,
					step = 0.1,
				},
				SetIconToBarHeight = {
					order = 450,
					type = "toggle",
					width = "full",
					name = L.options["Scale with bar height"],
					desc = L.options["Automatically resizes the icon when the bar height is adjusted"],
				},
				IconSize = {
					order = 500,
					type = "range",
					name = L.options["Icon Size"],
					desc = L.options["How big the icon is in width and height"],
					min = 10,
					max = 100,
					step = 1,
					disabled = function() return Alerts.db.profile.SetIconToBarHeight end,
				},
			},
		}

		bars_args.icon_group = icon_group

			local aurasanch_group = {
			type = "group",
			name = L.options["Auras Anchored Bars"],
			order = 600,
			args = {
				Auras_desc = {
					type = "header",
					name = L.options["Adjust settings related to the auras anchor"].."\n",
					order = 1,
				},
				AurasScale = {
					order = 100,
					type = "range",
					name = L.options["Bar Scale"],
					desc = L.options["Adjust the size of auras bars"],
					min = 0.5,
					max = 1.5,
					step = 0.05,
				},
				AurasAlpha = {
					type = "range",
					name = L.options["Bar Alpha"],
					desc = L.options["Adjust the transparency of auras bars"],
					order = 200,
					min = 0.1,
					max = 1,
					step = 0.05,
				},
				AurasBarWidth = {
					order = 300,
					type = "range",
					name = L.options["Bar Width"],
					desc = L.options["Adjust the width of auras bars"],
					min = 220,
					max = 1000,
					step = 1,
				},
				AurasGrowth = {
					order = 400,
					type = "select",
					name = L.options["Bar Growth"],
					desc = L.options["The direction auras bars grow"],
					values = {DOWN = L.options["Down"], UP = L.options["Up"]},
				},
				AurasTextWidth = {
					order = 500,
					type = "range",
					name = L.options["Text Width"],
					desc = L.options["The width of the text for auras bars"],
					min = 50,
					max = 1000,
					step = 1,
				},
				blan = genblank(500.1),
				blan2 = genblank(500.2),
				PixelPerfect_header = {
					type = "header",
					name = L.options["Pixel Perfect"],
					order = 510,
				},
				blank = genblank(520),
				blank2 = genblank(520.1),
				MessageAnchor_x = {
					type = "range",
					name = "X Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 530,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsAurasAnchor"]
						if pos then
							return pos.xOfs
						else
							return 312.4999134559484
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsAurasAnchor"].xOfs = v
						addon:LoadPosition("DXEAlertsAurasAnchor")
					end,							
				},
				MessageAnchor_y = {
					type = "range",
					name = "Y Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 540,
					width = "full",
					get = function(info)
						local pos = db.profile.Positions["DXEAlertsAurasAnchor"]
						if pos then
							return pos.yOfs
						else
							return 246.9998803907182
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsAurasAnchor"].yOfs = v
						addon:LoadPosition("DXEAlertsAurasAnchor")
					end,
				},
			},
		}
		bars_args.aurasanch_group = aurasanch_group
		
			local debuff_group = {
			type = "group",
			name = L.options["Debuff Anchored Bars"],
			order = 600,
			disabled = function() return not Alerts.db.profile.DebuffBars end,
			args = {
				Debuff_desc = {
					type = "header",
					name = L.options["Adjust settings related to the debuff anchor"].."\n",
					order = 1,
				},
				DebuffScale = {
					order = 100,
					type = "range",
					name = L.options["Bar Scale"],
					desc = L.options["Adjust the size of debuff bars"],
					min = 0.5,
					max = 1.5,
					step = 0.05,
				},
				DebuffAlpha = {
					type = "range",
					name = L.options["Bar Alpha"],
					desc = L.options["Adjust the transparency of debuff bars"],
					order = 200,
					min = 0.1,
					max = 1,
					step = 0.05,
				},
				DebuffBarWidth = {
					order = 300,
					type = "range",
					name = L.options["Bar Width"],
					desc = L.options["Adjust the width of debuff bars"],
					min = 220,
					max = 1000,
					step = 1,
				},
				DebuffGrowth = {
					order = 400,
					type = "select",
					name = L.options["Bar Growth"],
					desc = L.options["The direction debuff bars grow"],
					values = {DOWN = L.options["Down"], UP = L.options["Up"]},
				},
				DebuffTextWidth = {
					order = 500,
					type = "range",
					name = L.options["Text Width"],
					desc = L.options["The width of the text for debuff bars"],
					min = 50,
					max = 1000,
					step = 1,
				},
				blan = genblank(500.1),
				blan2 = genblank(500.2),
				PixelPerfect_header = {
					type = "header",
					name = L.options["Pixel Perfect"],
					order = 510,
				},
				blank = genblank(520),
				blank2 = genblank(520.1),
				MessageAnchor_x = {
					type = "range",
					name = "X Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 530,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsDebuffStackAnchor"]
						if pos then
							return pos.xOfs
						else
							return 312.4999134559484
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsDebuffStackAnchor"].xOfs = v
						addon:LoadPosition("DXEAlertsDebuffStackAnchor")
					end,							
				},
				MessageAnchor_y = {
					type = "range",
					name = "Y Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 540,
					width = "full",
					get = function(info)
						local pos = db.profile.Positions["DXEAlertsDebuffStackAnchor"]
						if pos then
							return pos.yOfs
						else
							return 20
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsDebuffStackAnchor"].yOfs = v
						addon:LoadPosition("DXEAlertsDebuffStackAnchor")
					end,
				},
			},
		}
		bars_args.debuff_group = debuff_group
		
		local top_group = {
			type = "group",
			name = L.options["Top Anchored Bars"],
			order = 600,
			disabled = function() return Alerts.db.profile.DisableDropdowns end,
			args = {
				top_desc = {
					type = "header",
					name = L.options["Adjust settings related to the top anchor"].."\n",
					order = 1,
				},
				TopScale = {
					order = 100,
					type = "range",
					name = L.options["Bar Scale"],
					desc = L.options["Adjust the size of top bars"],
					min = 0.5,
					max = 1.5,
					step = 0.05,
				},
				TopAlpha = {
					type = "range",
					name = L.options["Bar Alpha"],
					desc = L.options["Adjust the transparency of top bars"],
					order = 200,
					min = 0.1,
					max = 1,
					step = 0.05,
				},
				TopBarWidth = {
					order = 300,
					type = "range",
					name = L.options["Bar Width"],
					desc = L.options["Adjust the width of top bars"],
					min = 220,
					max = 1000,
					step = 1,
				},
				TopGrowth = {
					order = 400,
					type = "select",
					name = L.options["Bar Growth"],
					desc = L.options["The direction top bars grow"],
					values = {DOWN = L.options["Down"], UP = L.options["Up"]},
				},
				TopTextWidth = {
					order = 500,
					type = "range",
					name = L.options["Text Width"],
					desc = L.options["The width of the text for top bars"],
					min = 50,
					max = 1000,
					step = 1,
				},
				blank = genblank(550),
				blank44 = genblank(550.1),
				mid_desc = {
					type = "header",
					name = L.options["Settings"].."\n",
					order = 551,
				},
				blank234 = genblank(552),
				blank34 = genblank(552.1),
				AnimationSpeed = {
					order = 600,
					type = "range",
					name = L.options["Animation Speed"],
					desc = L.options["Control the animation speed when the bars moves from Top to Center."],
					min = 0.1,
					max = 1.5,
					step = 0.1,
					get = function(info)
						local var = info[#info]
						return db.profile.Globals[var]
					end,
					set = function(info,v,v2,v3,v4)
						local var = info[#info]
						db.profile.Globals[var] = v
						addon:RefreshProfile()
						addon.Alerts:RefreshProfile()
						--	addon["Notify"..var.."Changed"](addon,v)
					end,					
				},
				TransitionTime = {
					order = 700,
					type = "range",
					name = L.options["Flashtime"],
					desc = L.options["Control the default Flashtime in seconds left on alert when moving from Top to Center."],
					min = 1,
					max = 20,
					step = 1,
					get = function(info)
						local var = info[#info]
						return db.profile.Globals[var]
					end,
					set = function(info,v,v2,v3,v4)
						local var = info[#info]
						db.profile.Globals[var] = v
						addon:RefreshProfile()
						addon.Alerts:RefreshProfile()
						--	addon["Notify"..var.."Changed"](addon,v)
					end,					
				},	
				reset = {
						type = "execute",
						name = L.options["Reset"],
						func = function()
							db.profile.Globals.AnimationSpeed = 0.3
							db.profile.Globals.TransitionTime = 5
							addon:RefreshProfile()
							addon.Alerts:RefreshProfile()
							--	addon["Notify"..var.."Changed"](addon,v)
						end,
						order = 703,
					--	width = "half",
				},
				blan = genblank(800.1),
				blan2 = genblank(800.2),
				PixelPerfect_header = {
					type = "header",
					name = L.options["Pixel Perfect"],
					order = 810,
				},
				blank = genblank(820),
				blank2 = genblank(820.1),
				MessageAnchor_x = {
					type = "range",
					name = "X Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 830,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsTopStackAnchor"]
						if pos then
							return pos.xOfs
						else
							return -330.5000126174457
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsTopStackAnchor"].xOfs = v
						addon:LoadPosition("DXEAlertsTopStackAnchor")
					end,							
				},
				MessageAnchor_y = {
					type = "range",
					name = "Y Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 840,
					width = "full",
					get = function(info)
						local pos = db.profile.Positions["DXEAlertsTopStackAnchor"]
						if pos then
							return pos.yOfs
						else
							return 23.49995633474761
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsTopStackAnchor"].yOfs = v
						addon:LoadPosition("DXEAlertsTopStackAnchor")
					end,
				},
			},
		}
		bars_args.top_group = top_group

		local center_group = {
			type = "group",
			name = L.options["Center Anchored Bars"],
			order = 700,
			args = {
				center_desc = {
					type = "header",
					name = L.options["Adjust settings related to the center anchor"].."\n",
					order = 1,
				},
				CenterScale = {
					order = 100,
					type = "range",
					name = L.options["Bar Scale"],
					desc = L.options["Adjust the size of center bars"],
					min = 0.5,
					max = 1.5,
					step = 0.05,
				},
				CenterAlpha = {
					type = "range",
					name = L.options["Bar Alpha"],
					desc = L.options["Adjust the transparency of center bars"],
					order = 200,
					min = 0.1,
					max = 1,
					step = 0.05,
				},
				CenterBarWidth = {
					order = 300,
					type = "range",
					name = L.options["Bar Width"],
					desc = L.options["Adjust the width of center bars"],
					min = 220,
					max = 1000,
					step = 1,
				},
				CenterGrowth = {
					order = 400,
					type = "select",
					name = L.options["Bar Growth"],
					desc = L.options["The direction center bars grow"],
					values = {DOWN = L.options["Down"], UP = L.options["Up"]},
				},
				CenterTextWidth = {
					order = 500,
					type = "range",
					name = L.options["Text Width"],
					desc = L.options["The width of the text for center bars"],
					min = 50,
					max = 1000,
					step = 1,
				},
				blan = genblank(800.1),
				blan2 = genblank(800.2),
				PixelPerfect_header = {
					type = "header",
					name = L.options["Pixel Perfect"],
					order = 810,
				},
				blank = genblank(820),
				blank2 = genblank(820.1),
				MessageAnchor_x = {
					type = "range",
					name = "X Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 830,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsCenterStackAnchor"]
						if pos then
							return pos.xOfs
						else
							return 0
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsCenterStackAnchor"].xOfs = v
						addon:LoadPosition("DXEAlertsCenterStackAnchor")
					end,							
				},
				MessageAnchor_y = {
					type = "range",
					name = "Y Position",
					min = -2048,
					max = 2048,
					step = 1,
					order = 840,
					width = "full",
					get = function(info)
						local pos = db.profile.Positions["DXEAlertsCenterStackAnchor"]
						if pos then
							return pos.yOfs
						else
							return -235.9996233230851
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsCenterStackAnchor"].yOfs = v
						addon:LoadPosition("DXEAlertsCenterStackAnchor")
					end,
				},
			},
		}

		bars_args.center_group = center_group

		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end

			local intro_desc = L.options["You can fire local or raid bars. Local bars are only seen by you. Raid bars are seen by you and raid members; You have to be a raid officer to fire raid bars"]
			local howto_desc = L.options["Slash commands: |cffffff00/dxelb time text|r (local bar) or |cffffff00/dxerb time text|r (raid bar): |cffffff00time|r can be in the format |cffffd200minutes:seconds|r or |cffffd200seconds|r"]
			local example1 = "/dxerb 15 "..L.options["Pizza Timer"]
			local example2 = "/dxelb 6:00 "..L.options["Pizza Timer"]
			--local howto_desc = L.options["Slash commands: |cffffff00/lpull time text|r (local bar) or |cffffff00/pull time text|r (raid bar): |cffffff00time|r can be in the format |cffffd200minutes:seconds|r or |cffffd200seconds|r"]
			--local example1 = "/lpull 15 Pulling in..."
			--local example2 = "/pull 6:00 Pizza Timer"

			local custom_group = {
				type = "group",
				name = L.options["Custom Bars"],
				order = 750,
				args = {
					intro_desc = {
						type = "description",
						name = intro_desc,
						order = 1,
					},
					CustomLocalClr = {
						order = 2,
						type = "select",
						name = L.options["Local Bar Color"],
						desc = L.options["The color of local bars that you fire"],
						values = colors,
					},
					CustomRaidClr = {
						order = 3,
						type = "select",
						name = L.options["Raid Bar Color"],
						desc = L.options["The color of broadcasted raid bars fired by you or a raid member"],
						values = colors,
					},
					CustomSound = {
						order = 4,
						type = "select",
						name = L.options["Sound"],
						desc = L.options["The sound that plays when a custom bar is fired"],
						values = GetSounds,
					},
					CustomAudioCD = {
						type = "toggle",
						name = L.options["Audio Countdown"],
						order = 5,						
					},
					CustomAudiotime = {
						type = "range",
						name = L.options["Audio Time"],
						desc = L.options["Audio seconds voice counter"],
						order = 6,
						min = 3,
						max = 10,
						step = 1,						
						disabled = function() return not Alerts.db.profile.CustomAudioCD end,
					},	
					CustomBiggerBar = {
						order = 7,
						type = "toggle",
						name = L.options["Bigger Bar"],
						width = "normal",
					},
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 8,
						func = function()
							Alerts:DXELBTest()
						end,
					},
					howto_desc = {
						type = "description",
						name = "\n"..howto_desc,
						order = 9,
					},
					examples_desc = {
						type = "description",
						order = 10,
						name = "\n"..L.options["Examples"]..":\n\n   "..example1.."\n   "..example2,
					},				
				},
			}

			bars_args.custom_group = custom_group
		end
		
		----------- PULL
		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end

			local intro_desc = L.options["You can fire local or raid bars. Local bars are only seen by you. Raid bars are seen by you and raid members; You have to be a raid officer to fire raid bars"]
			local howto_desc = L.options["Slash commands: |cffffff00/lpull time text|r (local bar) or |cffffff00/pull time text|r (raid bar): |cffffff00time|r can be in the format |cffffd200minutes:seconds|r or |cffffd200seconds|r"]
			local example1 = "/lpull 15 "..L.options["Pulling in..."]
			local example2 = "/pull 15 "..L.options["Raid Pulling in..."]
			local pull_group = {
				type = "group",
				name = L.options["Pull Bar"],
				order = 750,
				args = {
					intro_desc = {
						type = "description",
						name = intro_desc,
						order = 1,
					},
					PullLocalClr = {
						order = 2,
						type = "select",
						name = L.options["Local Bar Color"],
						desc = L.options["The color of local bars that you fire"],
						values = colors,
					},
					PullRaidClr = {
						order = 3,
						type = "select",
						name = L.options["Raid Bar Color"],
						desc = L.options["The color of broadcasted raid bars fired by you or a raid member"],
						values = colors,
					},
					PullSound = {
						order = 4,
						type = "select",
						name = L.options["Sound"],
						desc = L.options["The sound that plays when a pull bar is fired"],
						values = GetSounds,
					},
					PullAudioCD = {
						type = "toggle",
						name = L.options["Audio Countdown"],
						order = 5,						
					},
					PullAudiotime = {
						type = "range",
						name = L.options["Audio Time"],
						desc = L.options["Audio seconds voice counter"],
						order = 6,
						min = 3,
						max = 10,
						step = 1,						
						disabled = function() return not Alerts.db.profile.PullAudioCD end,
					},	
					PullBiggerBar = {
						order = 7,
						type = "toggle",
						name = L.options["Bigger Bar"],
						width = "normal",
					},	
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 8,
						func = function()
							Alerts:PullTest()
						end,
					},
					howto_desc = {
						type = "description",
						name = "\n"..howto_desc,
						order = 9,
					},
					examples_desc = {
						type = "description",
						order = 10,
						name = "\n"..L.options["Examples"]..":\n\n   "..example1.."\n   "..example2,
					},
				},
			}

			bars_args.pull_group = pull_group
		end
		
		----------- BREAK
		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end

			local intro_desc = L.options["You can fire local or raid bars. Local bars are only seen by you. Raid bars are seen by you and raid members; You have to be a raid officer to fire raid bars"]
			local howto_desc = L.options["Slash commands: |cffffff00/lbreak time text|r (local bar) or |cffffff00/break time text|r (raid bar): |cffffff00time|r can be in the format |cffffd200minutes:seconds|r or |cffffd200seconds|r"]
			local example1 = "/lbreak 15 "..L.options["Break for Coffe"]
			local example2 = "/break 15 "..L.options["Raid Break for bathroom"]

			local break_group = {
				type = "group",
				name = L.options["Break Bar"],
				order = 750,
				args = {
					intro_desc = {
						type = "description",
						name = intro_desc,
						order = 1,
					},
					BreakLocalClr = {
						order = 2,
						type = "select",
						name = L.options["Local Bar Color"],
						desc = L.options["The color of local bars that you fire"],
						values = colors,
					},
					BreakRaidClr = {
						order = 3,
						type = "select",
						name = L.options["Raid Bar Color"],
						desc = L.options["The color of broadcasted raid bars fired by you or a raid member"],
						values = colors,
					},
					BreakSound = {
						order = 4,
						type = "select",
						name = L.options["Sound"],
						desc = L.options["The sound that plays when a break bar is fired"],
						values = GetSounds,
					},
					BreakAudioCD = {
						type = "toggle",
						name = L.options["Audio Countdown"],
						order = 5,						
					},
					BreakAudiotime = {
						type = "range",
						name = L.options["Audio Time"],
						desc = L.options["Audio seconds voice counter"],
						order = 6,
						min = 3,
						max = 10,
						step = 1,						
						disabled = function() return not Alerts.db.profile.BreakAudioCD end,
					},	
					BreakBiggerBar = {
						order = 7,
						type = "toggle",
						name = L.options["Bigger Bar"],
						width = "normal",
					},					
					test = {
						type = "execute",
						name = L.options["Test"],
						order = 8,
						func = function()
							Alerts:BreakTest()
						end,
					},
					howto_desc = {
						type = "description",
						name = "\n"..howto_desc,
						order = 9,
					},
					examples_desc = {
						type = "description",
						order = 10,
						name = "\n"..L.options["Examples"]..":\n\n   "..example1.."\n   "..example2,
					},				
				},
			}

			bars_args.break_group = break_group
		end
		
		----------- Fastest Kill
		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end
			local FastestKill_group = {
				type = "group",
				name = L.options["Fastest Kill Bar"],
				order = 750,
				args = {
					blank2 = genblank(0.5),
					intro_desc = {
						type = "description",
						name = L.options["Here you can configure settings related to boss fastest kills. If you haven't yet killed that boss then no bar will show"],
						order = 1,
					},				
					blank2 = genblank(1.5),
					--blank2 = genblank(1.6),
					KEnabled = {
						order = 2,
						type = "toggle",
						name = L.options["Enable"],
						width = "normal",
						get = function(info) return db.profile.Kills.KEnabled end,
						set = function(info,v) db.profile.Kills.KEnabled = v end,
					},
					--blank2 = genblank(8.5),
						sett_desc = {
							type = "header",
							name = L.options["Settings"],
							order = 2.1,
						},
				--	blank2 = genblank(9.5),

					--[[resp_group = {
						type = "group",
						name = L.options["Settings"],
						inline = true,
						order = 3,
						args = {--]]
							BiggerBar = {
								order = 18,
								type = "toggle",
								name = L.options["Bigger Bar"],
								width = "normal",
								get = function(info) return db.profile.Kills.BiggerBar end,
								set = function(info,v) db.profile.Kills.BiggerBar = v end,
								disabled = function() return not db.profile.Kills.KEnabled end,		
							},
							BarCrl = {
								order = 11,
								type = "select",
								name = L.options["Bar Color"],
								desc = L.options["The color of statistic bar"],
								values = colors,
								get = function(info) return db.profile.Kills.BarCrl end,
								set = function(info,v) db.profile.Kills.BarCrl = v end,
								disabled = function() return not db.profile.Kills.KEnabled end,
							},	
							test = {
								type = "execute",
								name = L.options["Test"],
								order = 12,
								width = "normal",
								func = function()
									local a = db.profile.Kills.BarCrl
									local b = db.profile.Kills.BiggerBar
									Alerts:StatisticsTest(a,b)
								end,
								disabled = function() return not db.profile.Kills.KEnabled end,						
							},			
					--	},
					--},											
				},
			}

			bars_args.FastestKill_group = FastestKill_group
		end

		----------- Respawn
		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end
			local Respawn_group = {
				type = "group",
				name = L.options["Respawn Bar"],
				order = 751,
				args = {
					blank2 = genblank(0.5),
					intro_desc = {
						type = "description",
						name = L.options["Here you can configure settings related to respawn bar. This bar show after you wipe on a boss displaying the time until the boss respawns"],
						order = 1,
					},				
					blank2 = genblank(1.5),
					--blank2 = genblank(1.6),
					Enabled = {
						order = 2,
						type = "toggle",
						name = L.options["Enable"],
						width = "normal",
						get = function(info) return db.profile.RespawnBar.Enabled end,
						set = function(info,v) db.profile.RespawnBar.Enabled = v end,
					},
						sett_desc = {
							type = "header",
							name = L.options["Settings"],
							order = 2.1,
						},
					--[[resp_group = {
						type = "group",
						name = L.options["Settings"],
						inline = true,
						order = 3,
						args = {--]]
							BiggerBar = {
								order = 18,
								type = "toggle",
								name = L.options["Bigger Bar"],
								width = "normal",
								get = function(info) return db.profile.RespawnBar.BiggerBar end,
								set = function(info,v) db.profile.RespawnBar.BiggerBar = v end,
								disabled = function() return not db.profile.RespawnBar.Enabled end,		
							},
							BarColor = {
								order = 11,
								type = "select",
								name = L.options["Bar Color"],
								values = colors,
								get = function(info) return db.profile.RespawnBar.BarColor end,
								set = function(info,v) db.profile.RespawnBar.BarColor = v end,
								disabled = function() return not db.profile.RespawnBar.Enabled end,
							},	
							test = {
								type = "execute",
								name = L.options["Test"],
								order = 12,
								width = "normal",
								func = function()
									local a = db.profile.RespawnBar.BarColor
									local b = db.profile.RespawnBar.BiggerBar
									Alerts:RespawnTest(a,b)
								end,
								disabled = function() return not db.profile.RespawnBar.Enabled end,						
							},									
					--	},
					--},											
				},
			}

			bars_args.Respawn_group = Respawn_group
		end
		
		----------- Enrage
		do
			local colors = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
			end
			local Enrage_group = {
				type = "group",
				name = L.options["Enrage Bar"],
				order = 751,
				args = {
					blank2 = genblank(0.5),
					intro_desc = {
						type = "description",
						name = L.options["Here you can configure settings related to enrage bar. If a encounter still doesn't have a enrage timer know then no enrage bar will show"],
						order = 1,
					},				
					blank2 = genblank(1.5),
					--blank2 = genblank(1.6),
					Enabled = {
						order = 2,
						type = "toggle",
						name = L.options["Enable"],
						width = "normal",
						get = function(info) return db.profile.Enrage.Enabled end,
						set = function(info,v) db.profile.Enrage.Enabled = v end,
					},
					--[[resp_group = {
						type = "group",
						name = L.options["Settings"],
						inline = true,
						order = 3,
						args = {--]]
						sett_desc = {
							type = "header",
							name = L.options["Settings"],
							order = 2.1,
						},
						--blank = genblank(2.5),
							BarColor = {
								order = 11,
								type = "select",
								name = L.options["Bar Color"],
								values = colors,
								get = function(info) return db.profile.Enrage.BarColor end,
								set = function(info,v) db.profile.Enrage.BarColor = v end,
								disabled = function() return not db.profile.Enrage.Enabled end,
							},
							FlashBarColor = {
								order = 12,
								type = "select",
								name = L.options["Flash Color"],
								values = colors,
								get = function(info) return db.profile.Enrage.FlashBarColor end,
								set = function(info,v) db.profile.Enrage.FlashBarColor = v end,
								disabled = function() return not db.profile.Enrage.Enabled end,
							},
							Flashtime = {
								type = "range",
								name = L.options["Flashtime"],
								desc = L.options["Flashtime in seconds left on alert"],
								order = 13,
								--width = "half",
								min = 1,
								max = 20,
								step = 1,
								disabled = function() return not db.profile.Enrage.Enabled end,
								get = function(info) return db.profile.Enrage.Flashtime end,
								set = function(info,v) db.profile.Enrage.Flashtime = v end,
							},
							AudioCd = {
								type = "toggle",
								name = L.options["Audio Countdown"],
								order = 14,
								get = function(info) return db.profile.Enrage.AudioCd end,
								set = function(info,v) db.profile.Enrage.AudioCd = v end,								
								disabled = function() return not db.profile.Enrage.Enabled end,
							},
							Audiotime = {
								type = "range",
								name = L.options["Audio Time"],
								desc = L.options["Audio seconds voice counter"],
								order = 15,
								min = 3,
								max = 10,
								step = 1,
								get = function(info) return db.profile.Enrage.Audiotime end,
								set = function(info,v) db.profile.Enrage.Audiotime = v end,								
								disabled = function() return not db.profile.Enrage.Enabled or not db.profile.Enrage.AudioCd end,
							},						
							test = {
								type = "execute",
								name = L.options["Test"],
								order = 16,
								width = "normal",
								func = function()
									local a = db.profile.Enrage.BarColor
									local b = db.profile.Enrage.FlashBarColor
									local c = db.profile.Enrage.Flashtime
									local d = db.profile.Enrage.AudioCd
									local e = db.profile.Enrage.Audiotime
									local f = db.profile.Enrage.BiggerBar
									Alerts:EnrageTest(a,b,c,d,e,f)
								end,
								disabled = function() return not db.profile.Enrage.Enabled end,						
							},
							BiggerBar = {
								order = 17,
								type = "toggle",
								name = L.options["Bigger Bar"],
								width = "normal",
								get = function(info) return db.profile.Enrage.BiggerBar end,
								set = function(info,v) db.profile.Enrage.BiggerBar = v end,
								disabled = function() return not db.profile.Enrage.Enabled end,		
							},							
						},
				--	},	
				--},
			}

			bars_args.Enrage_group = Enrage_group
		end
		
		-- WARNINGS
		local warning_bar_group = {
			type = "group",
			name = L.options["Warning Bars"],
			order = 800,
			disabled = function() return Alerts.db.profile.EnableWarningMessage end,
			args = {
				WarningBars = {
					type = "toggle",
					order = 100,
					name = L.options["Enable Warning Bars"],
					set = SetNoRefresh,
				},
				warning_bars = {
					type = "group",
					name = L.options["Warning Anchor"],
					order = 200,
					inline = true,
					disabled = function() return not Alerts.db.profile.WarningBars end,
					args = {
						WarningAnchor = {
							order = 100,
							type = "toggle",
							name = L.options["Enable Warning Anchor"],
							desc = L.options["Anchors all warning bars to the warning anchor instead of the center anchor"],
							width = "full",
						},
					}
				},
			},
		}

		bars_args.warning_bar_group = warning_bar_group

		do
			local warning_bars_args = warning_bar_group.args.warning_bars.args

			local warning_settings_group = {
				type = "group",
				name = "",
				order = 300,
				disabled = function() return not Alerts.db.profile.WarningAnchor or not Alerts.db.profile.WarningBars end,
				args = {
					WarningScale = {
						order = 100,
						type = "range",
						name = L.options["Bar Scale"],
						desc = L.options["Adjust the size of warning bars"],
						min = 0.5,
						max = 1.5,
						step = 0.05,
					},
					WarningAlpha = {
						type = "range",
						name = L.options["Bar Alpha"],
						desc = L.options["Adjust the transparency of warning bars"],
						order = 200,
						min = 0.1,
						max = 1,
						step = 0.05,
					},
					WarningBarWidth = {
						order = 300,
						type = "range",
						name = L.options["Bar Width"],
						desc = L.options["Adjust the width of warning bars"],
						min = 220,
						max = 1000,
						step = 1,
					},
					WarningTextWidth = {
						order = 350,
						type = "range",
						name = L.options["Text Width"],
						desc = L.options["The width of the text for warning bars"],
						min = 50,
						max = 1000,
						step = 1,
					},
					WarningGrowth = {
						order = 400,
						type = "select",
						name = L.options["Bar Growth"],
						desc = L.options["The direction warning bars grow"],
						values = {DOWN = L.options["Down"], UP = L.options["Up"]},
					},
					RedirectCenter = {
						order = 500,
						type = "toggle",
						name = L.options["Redirect center bars"],
						desc = L.options["Anchor a center bar to the warnings anchor if its duration is less than or equal to threshold time"],
						width = "full",
					},
					RedirectThreshold = {
						order = 600,
						type = "range",
						name = L.options["Threshold time"],
						desc = L.options["If a center bar's duration is less than or equal to this then it anchors to the warnings anchor"],
						min = 1,
						max = 15,
						step = 1,
						disabled = function() return not Alerts.db.profile.WarningBars or not Alerts.db.profile.WarningAnchor or not Alerts.db.profile.RedirectCenter end
					},
				},
			}
			warning_bars_args.warning_settings_group = warning_settings_group
		end

		local warning_message_group = {
			type = "group",
			name = L.options["Warning Messages"],
			order = 140,
			args = {
				WarningMessages = {
					type = "toggle",
					name = L.options["Enable Warning Messages"],
					desc = L.options["Output to an additional interface"],
					order = 1,
					width = "full",
				},
				inner_group = {
					type = "group",
					name = "",
					disabled = function() return not Alerts.db.profile.WarningMessages end,
					inline = true,
					args = {
						warning_desc = {
							type = "description",
							name = L.options["Alerts are split into three categories: cooldowns, durations, and warnings. Cooldown and duration alerts can fire a message before they end and when they popup. Warning alerts can only fire a message when they popup. Alerts suffixed self can only fire a popup message even if it is a duration"].."\n",
							order = 2,
						},
						AnnounceToRaid = {
							order = 2.5,
							type = "toggle",
							name = L.options["Announce to raid"],
							desc = L.options["Announce warning messages through raid warning chat"],
							width = "full",
						},
						ClrWarningText = {
							order = 3,
							type = "toggle",
							name = L.options["Color Text"],
							desc = L.options["Class colors text"],
							width = "full",
						},
						SinkIcon = {
							order = 4,
							type = "toggle",
							name = L.options["Show Icon"],
							desc = L.options["Display an icon to the left of a warning message"],
							width = "full",
						},
						BeforeThreshold = {
							order = 5,
							type = "range",
							name = L.options["Before End Threshold"],
							desc = L.options["How many seconds before an alert ends to fire a warning message. This only applies to cooldown and duration type alerts"],
							min = 1,
							max = 15,
							step = 1,
						},
						filter_group = {
							type = "group",
							order = 6,
							name = L.options["Show messages for"].."...",
							inline = true,
							args = {
								filter_desc = {
									type = "description",
									name = L.options["Enabling |cffffd200X popups|r will make it fire a message on appearance. Enabling |cffffd200X before ending|r will make it fire a message before ending based on before end threshold"],
									order = 1,
								},
								CdPopupMessage = {type = "toggle", name = L.options["Cooldown popups"], order = 2, width = "full"},
								CdBeforeMessage = {type = "toggle", name = L.options["Cooldowns before ending"], order = 3, width = "full"},
								DurPopupMessage = {type = "toggle", name = L.options["Duration popups"], order = 4, width = "full"},
								DurBeforeMessage = {type = "toggle", name = L.options["Durations before ending"], order = 5, width = "full"},
								WarnPopupMessage = {type = "toggle", name = L.options["Warning popups"], order = 6, width = "full"},
							}
						},
						output_desc = {
							type = "description",
							order = 7,
							name = L.options["You can output warning messages many different ways and they will only be seen by you. If you don't want to see these then click |cffffd200'None'|r"],
						},
						Output = Alerts:GetSinkAce3OptionsDataTable(),
					},
				},
			},
		}
		alerts_args.warning_message_group = warning_message_group
		warning_message_group.args.inner_group.args.Output.disabled = function() return not Alerts.db.profile.WarningMessages end
		warning_message_group.args.inner_group.args.Output.inline = true
		warning_message_group.args.inner_group.args.Output.order = -1

		local flash_group = {
			type = "group",
			name = L.options["Screen Flash"],
			order = 200,
			args = {
				flash_desc = {
					type = "description",
					name = L.options["The color of the flash becomes the main color of the alert. Colors for each alert are set in the Encounters section. If the color is set to 'Clear' it defaults to black"].."\n",
					order = 50,
				},
				DisableScreenFlash = {
					order = 75,
					type = "toggle",
					name = L.options["Disable Screen Flash"],
					desc = L.options["Turns off all alert screen flashes"],
					set = SetNoRefresh,
					width = "full",
				},
				flash_inner_group = {
					name = "",
					type = "group",
					order = 100,
					inline = true,
					disabled = function() return Alerts.db.profile.DisableScreenFlash end,
					args = {
						FlashTest = {
							type = "execute",
							name = L.options["Test Flash"],
							desc = L.options["Fires a flash using a random color"],
							order = 100,
							func = "FlashTest",
						},
						FlashTexture = {
							type = "select",
							name = L.options["Texture"],
							desc = L.options["Select a background texture"],
							order = 120,
							values = Alerts.FlashTextures,
							set = function(info,v) Alerts.db.profile.FlashTexture = v; Alerts:UpdateFlashSettings() end,
						},
						FlashAlpha = {
							type = "range",
							name = L.options["Alpha"],
							desc = L.options["Adjust the transparency of the flash"],
							order = 200,
							min = 0.1,
							max = 1,
							step = 0.05,
						},
						FlashDuration = {
							type = "range",
							name = L.options["Duration"],
							desc = L.options["Adjust how long the flash lasts"],
							order = 300,
							min = 0.2,
							max = 3,
							step = 0.05,
						},
						EnableOscillations = {
							type = "toggle",
							name = L.options["Enable Oscillations"],
							desc = L.options["Enables the strobing effect"],
							order = 350,
						},
						FlashOscillations = {
							type = "range",
							name = L.options["Oscillations"],
							desc = L.options["Adjust how many times the flash fades in and out"],
							order = 400,
							min = 1,
							max = 10,
							step = 1,
							disabled = function() return Alerts.db.profile.DisableScreenFlash or not Alerts.db.profile.EnableOscillations end,
						},
						blank = genblank(450),
						ConstantClr = {
							type = "toggle",
							name = L.options["Use Constant Color"],
							desc = L.options["Make the screen flash always be the global color. It will not become the main color of the alert."],
							order = 500,
						},
						GlobalColor = {
							type = "color",
							name = L.options["Global Color"],
							order = 600,
							disabled = function() return Alerts.db.profile.DisableScreenFlash or not Alerts.db.profile.ConstantClr end,
						},
					},
				},
			},
		}

		alerts_args.flash_group = flash_group

		local colors = {}
		for k,c in pairs(addon.Media.Colors) do
			local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
			colors[k] = hex
		end
		local msg_anchor_group = {
			type = "group",
			name = L.options["Messages"],
			order = 101,
			childGroups = "tab",
			set = function(info,v)
				db.profile.Enrage[info[#info]] = v
			end,
			get = function(info)
				return db.profile.Enrage[info[#info]]
			end,
			args = { },
		}
		
		alerts_args.msg_anchor_group = msg_anchor_group
		local msg_args = msg_anchor_group.args
		
		local message_anchor_group = {
			type = "group",
			name = L.options["Message Anchor"],
			order = 1,
			set = function(info,v) db.profile.MessageAnchor[info[#info]] = v end,
			get = function(info) return db.profile.MessageAnchor[info[#info]] end,
			args = {
				message_anchor_header = {
					type = "header",
					name = L.options["Adjust settings related to the message anchor"],
					order = 1,
				},
				blank = genblank(2),
				blank2 = genblank(2.1),
				messagescale = {
					order = 3,
					type = "range",
					name = L.options["Message Scale"],
					desc = L.options["Adjust the size of message anchor"],
					min = 0.5,
					max = 1.5,
					step = 0.05,
				},
				messageholdtime = {
					order = 4,
					type = "range",
					name = L.options["Message HoldTime"],
					desc = L.options["Adjust the time of the messages stay on screen"],
					min = 1,
					max = 5,
					step = 1,
				},
				messageShowLeftIcon = {
					order = 5,
					type = "toggle",
					name = L.options["Show Left Icon"],
					desc = L.options["Shows a spell icon on the left side of the message"],
				},
				messageShowRightIcon = {
					order = 6,
					type = "toggle",
					name = L.options["Show Right Icon"],
					desc = L.options["Shows a spell icon on the right side of the message"],
				},
				--blank = genblank(6),

				blank = genblank(8),
				messagefadeintime = {
					order = 9,
					type = "range",
					name = L.options["FadeIn Time"],
					desc = L.options["Adjust the animation of the message appearing on screen"],
					min = 0.2,
					max = 1,
					step = 0.05,
				},
				messagefadeouttime = {
					order = 10,
					type = "range",
					name = L.options["FadeOut Time"],
					desc = L.options["Adjust the animation of the message disappearing on screen"],
					min = 0.2,
					max = 1,
					step = 0.05,
				},
				MessageFlags = {
					order = 12,
					type = "select",
					name = L.options["Font Quality"],
					desc = L.options["The message font quality"],
					values = {OUTLINE = L.options["OUTLINE"], THICKOUTLINE = L.options["THICKOUTLINE"]},			
				},
				TestMessage = {
					name = L.options["Test Message"],
					type = "execute",
					order = 12.5,
					--width = "double",
					--desc = L.options["Displays all arrows and then rotates them for ten seconds"],
					func = function()
						addon.Alerts:NewTMessage(L.options["Testing Message"],"RED",addon.ST[28374],"ALERT13")
					end,
				},
				MessageSize = {
					order = 13,
					type = "range",
					name = L.options["Font Size"],
					desc = L.options["Select a font size used on Message Anchor"],
					min = 8,
					max = 20,
					step = 1,
				},
				MessageFont = {
					order = 14,
					type = "select",
					name = L.options["Message Font"],
					desc = L.options["Font used on Message Anchor"],
					values = SM:HashTable("font"),
					dialogControl = "LSM30_Font",
					},						
				MessageGrowth = {
					order = 15,
					type = "select",
					name = L.options["Message Growth"],
					desc = L.options["The direction Message grow"],
					values = {DOWN = L.options["Down"], UP = L.options["Up"]},			
				},
				blan = genblank(15.1),
				blan2 = genblank(15.2),
				PixelPerfect_header = {
					type = "header",
					name = L.options["Pixel Perfect"],
					order = 16,
				},
				blank = genblank(17),
				blank2 = genblank(17.1),
				MessageAnchor_x = {
					type = "range",
					name = L.options["X Position"],
					min = -2048,
					max = 2048,
					step = 1,
					order = 18,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsMessageStackAnchor"]
						if pos then
							return pos.xOfs
						else
							return 0
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsMessageStackAnchor"].xOfs = v
						addon:LoadPosition("DXEAlertsMessageStackAnchor")
					end,
				},
				MessageAnchor_y = {
					type = "range",
					name = L.options["Y Position"],
					min = -2048,
					max = 2048,
					step = 1,
					order = 19,
					width = "full",
					get = function(info) 
						local pos = db.profile.Positions["DXEAlertsMessageStackAnchor"]
						if pos then
							return pos.yOfs
						else
							return 21.00003180006635
						end
					end,
					set = function(info,v)
						db.profile.Positions["DXEAlertsMessageStackAnchor"].yOfs = v
						addon:LoadPosition("DXEAlertsMessageStackAnchor")
					end,									
				},
			},
		}
		msg_args.message_anchor_group = message_anchor_group
		
		local warning_anchor_group = {
			type = "group",
			name = L.options["Warning Anchor"],
			order = 2,
			set = function(info,v)
				if info[#info] == "TopMessageSize" or info[#info] == "TopMessageFont" or info[#info] == "TopMessageFlags" or info[#info] == "TopMessageShowLeftIcon" or info[#info] == "TopMessageShowRightIcon" then
					db.profile.TopMessageAnchor[info[#info]] = v
				end
			end,
			get = function(info)
				if info[#info] == "TopMessageSize" or info[#info] == "TopMessageFont" or info[#info] == "TopMessageFlags" or info[#info] == "TopMessageShowLeftIcon" or info[#info] == "TopMessageShowRightIcon" then
					return db.profile.TopMessageAnchor[info[#info]]
				end
			end,
			args = {
				--[[topmessage_group = {
					type = "group",
					order = 3,
					name = L.options["Adjust settings related to the top warning message anchor"],
					inline = true,
					args = {--]]
						topmessage_anchor_header = {
							type = "header",
							name = L.options["Adjust settings related to the top warning message anchor"],
							order = 16,
						},
						TopMessageSize = {
							order = 17,
							type = "range",
							name = L.options["Font Size"],
							desc = L.options["Select a font size used on warning message"],
							min = 10,
							max = 35,
							step = 1,
						},
						TopMessageFont = {
							order = 18,
							type = "select",
							name = L.options["Message Font"],
							desc = L.options["Font used on Alert Anchor"],
							values = SM:HashTable("font"),
							dialogControl = "LSM30_Font",
						},
						TopMessageFlags = {
							order = 19,
							type = "select",
							name = L.options["Font Quality"],
							desc = L.options["The message font quality"],
							values = {OUTLINE = L.options["OUTLINE"], THICKOUTLINE = L.options["THICKOUTLINE"]},			
						},
						TestAlert = {
							name = L.options["Test"],
							type = "execute",
							order = 20,
							func = function()
								addon:NewWarning(L.options["Testing Message Alert"],"RED",3,addon.ST[136769])
							end,
						},
						TopMessageShowLeftIcon = {
							order = 21,
							type = "toggle",
							name = L.options["Show Left Icon"],
							desc = L.options["Shows a spell icon on the left side of the message"],
						},
						TopMessageShowRightIcon = {
							order = 22,
							type = "toggle",
							name = L.options["Show Right Icon"],
							desc = L.options["Shows a spell icon on the right side of the message"],
						},
						blan = genblank(22.1),
						blan2 = genblank(22.2),
						PixelPerfect_header = {
							type = "header",
							name = L.options["Pixel Perfect"],
							order = 23,
						},
						blank = genblank(24),
						blank2 = genblank(24.1),
						MessageAnchor_x = {
							type = "range",
							name = L.options["X Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 25,
							width = "full",
							get = function(info) 
								local pos = db.profile.Positions["DXEAlertsWarningStackAnchor"]
								if pos then
									return pos.xOfs
								else
									return 0
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEAlertsWarningStackAnchor"].xOfs = v
								addon:LoadPosition("DXEAlertsWarningStackAnchor")
							end,							
						},
						MessageAnchor_y = {
							type = "range",
							name = L.options["Y Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 26,
							width = "full",
							get = function(info)
								local pos = db.profile.Positions["DXEAlertsWarningStackAnchor"]
								if pos then
									return pos.yOfs
								else
									return 246.9998803907182
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEAlertsWarningStackAnchor"].yOfs = v
								addon:LoadPosition("DXEAlertsWarningStackAnchor")
							end,
						},
				--	},
				--},
			},
		}
		msg_args.warning_anchor_group = warning_anchor_group
		
		local inform_anchor_group = {
			type = "group",
			name = L.options["Inform Anchor"],
			order = 3,
			set = function(info,v)
				if info[#info] == "InformMessageSize" or info[#info] == "InformMessageFont" or info[#info] == "InformMessageFlags" or info[#info] == "InformMessageShowLeftIcon" or info[#info] == "InformMessageShowRightIcon" then
					db.profile.InformMessageAnchor[info[#info]] = v
				end
			end,
			get = function(info)
				if info[#info] == "InformMessageSize" or info[#info] == "InformMessageFont" or info[#info] == "InformMessageFlags" or info[#info] == "InformMessageShowLeftIcon" or info[#info] == "InformMessageShowRightIcon" then
					return db.profile.InformMessageAnchor[info[#info]]
				end
			end,
			args = {
				--[[informmessage_group = {
					type = "group",
					order = 3,
					name = L.options["Adjust settings related to the inform warning message anchor"],
					inline = true,
					args = {--]]
						informmessage_anchor_header = {
							type = "header",
							name = L.options["Adjust settings related to the inform warning message anchor"],
							order = 22,
						},
						InformMessageSize = {
							order = 23,
							type = "range",
							name = L.options["Font Size"],
							desc = L.options["Select a font size used on warning message"],
							min = 10,
							max = 35,
							step = 1,
						},
						InformMessageFont = {
							order = 24,
							type = "select",
							name = L.options["Message Font"],
							desc = L.options["Font used on Inform Anchor"],
							values = SM:HashTable("font"),
							dialogControl = "LSM30_Font",
						},						
						InformMessageFlags = {
							order = 25,
							type = "select",
							name = L.options["Font Quality"],
							desc = L.options["The message font quality"],
							values = {OUTLINE = L.options["OUTLINE"], THICKOUTLINE = L.options["THICKOUTLINE"]},			
						},
						TestInform = {
							name = L.options["Test"],
							type = "execute",
							order = 26,
							func = function()
								addon:InformWarning(L.options["Testing Message Inform"],"MIDBLUE",3,addon.ST[134926])
							end,
						},	
						InformMessageShowLeftIcon = {
							order = 27,
							type = "toggle",
							name = L.options["Show Left Icon"],
							desc = L.options["Shows a spell icon on the left side of the message"],
						},
						InformMessageShowRightIcon = {
							order = 28,
							type = "toggle",
							name = L.options["Show Right Icon"],
							desc = L.options["Shows a spell icon on the right side of the message"],
						},	
						blan = genblank(28.1),
						blan2 = genblank(28.2),
						PixelPerfect_header = {
							type = "header",
							name = L.options["Pixel Perfect"],
							order = 29,
						},
						blank = genblank(30),
						blank2 = genblank(30.1),
						MessageAnchor_x = {
							type = "range",
							name = "X Position",
							min = -2048,
							max = 2048,
							step = 1,
							order = 31,
							width = "full",
							get = function(info) 
								local pos = db.profile.Positions["DXEAlertsInformStackAnchor"]
								if pos then
									return pos.xOfs
								else
									return 0
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEAlertsInformStackAnchor"].xOfs = v
								addon:LoadPosition("DXEAlertsInformStackAnchor")
							end,							
						},
						MessageAnchor_y = {
							type = "range",
							name = "Y Position",
							min = -2048,
							max = 2048,
							step = 1,
							order = 32,
							width = "full",
							get = function(info)
								local pos = db.profile.Positions["DXEAlertsInformStackAnchor"]
								if pos then
									return pos.yOfs
								else
									return -60
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEAlertsInformStackAnchor"].yOfs = v
								addon:LoadPosition("DXEAlertsInformStackAnchor")
							end,
						},							
				--	},
				--},
			},
		}
		msg_args.inform_anchor_group = inform_anchor_group

		local enrage_group = {
			type = "group",
			name = L.options["Enrage"],
			order = 10,
			args = {
				blank = genblank(100),
				header = {
					type = "description",
					name = L.options["Here you can configure settings related to enrage messages. This messages will display across the encounter depending on boss enrage time, if a encounter still doesn't have a enrage timer know then no messages will display"],
					order = 0.1,
					width = "full",
				},
				blank2 = genblank(0.2),
				--[[header1_group = {
					type = "group",
					name = L.options["Settings"],					
					order = 203,
					inline = true,
					args = {--]]
						r1_anchor_header = {
							type = "header",
							name = L.options["Beginning"], 
							order = 1,
						},					
						EnableMessage = {
							order = 2,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},					
						MessageColor = {
							order = 4,
							type = "select",
							name = L.options["Messages Color"],
							--desc = L.options["The color of enrage bar"],
							values = colors,
							disabled = function() return not db.profile.Enrage.EnableMessage end,
						},
						Audio = {
							order = 5,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.EnableMessage end,
						},
						test = {
							type = "execute",
							name = L.options["Test"],
							order = 5.5,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.MessageColor
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.Audio
								Alerts:NewTMessage(GetSpellInfo(8599).." ("..SecondsToTime(600)..")",a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.EnableMessage end,						
						},				
						blank2 = genblank(5.6),
						r2_anchor_header = {
							type = "header",
							name = L.options["Half Time"],
							order = 6,
						},
						EnabledHalf = {
							order = 7,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},
						MessageHalfColor = {
							order = 8,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							disabled = function() return not db.profile.Enrage.EnabledHalf end,
						},								
						AudioHalf = {
							order = 9,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.EnabledHalf end,
						},						
						test2 = {
							type = "execute",
							name = L.options["Test"],
							order = 14,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.MessageHalfColor
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.AudioHalf
								Alerts:NewTMessage(GetSpellInfo(8599).." ("..SecondsToTime(300)..")",a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.EnabledHalf end,						
						},
						blank3 = genblank(15.6),
						r3_anchor_header = {
							type = "header",
							name = SecondsToTime(60),
							order = 16,
						},
						Enable1m = {
							order = 17,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},
						Message1mColor = {
							order = 18,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							disabled = function() return not db.profile.Enrage.Enable1m end,
						},								
						Audio1m = {
							order = 19,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.Enable1m end,
						},						
						test3 = {
							type = "execute",
							name = L.options["Test"],
							order = 20,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.Message1mColor
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.Audio1m
								Alerts:NewTMessage(GetSpellInfo(8599).." ("..SecondsToTime(60)..")",a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.Enable1m end,						
						},
						blank4 = genblank(20.6),
						r4_anchor_header = {
							type = "header",
							name = SecondsToTime(30),
							order = 21,
						},
						Enable30 = {
							order = 22,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},
						Message30Color = {
							order = 23,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							disabled = function() return not db.profile.Enrage.Enable30 end,
						},								
						Audio30 = {
							order = 24,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.Enable30 end,
						},						
						test4 = {
							type = "execute",
							name = L.options["Test"],
							order = 25,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.Message30Color
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.Audio30
								Alerts:NewTMessage(GetSpellInfo(8599).." ("..SecondsToTime(30)..")",a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.Enable30 end,						
						},
						blank5 = genblank(25.6),
						r5_anchor_header = {
							type = "header",
							name = SecondsToTime(15),
							order = 25.8,
						},
						Enable15 = {
							order = 26,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},
						Message15Color = {
							order = 27,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							disabled = function() return not db.profile.Enrage.Enable15 end,
						},								
						Audio15 = {
							order = 28,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.Enable15 end,
						},						
						test5 = {
							type = "execute",
							name = L.options["Test"],
							order = 29,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.Message15Color
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.Audio15
								Alerts:NewTMessage(GetSpellInfo(8599).." ("..SecondsToTime(15)..")",a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.Enable15 end,						
						},	
						blank6 = genblank(29.6),
						r6_anchor_header = {
							type = "header",
							name = L.options["Enrage"],
							order = 30,
						},
						EnableEnrage = {
							order = 31,
							type = "toggle",
							name = L.options["Enable"],
							width = "half",
						},
						MessageEnrageColor = {
							order = 32,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							disabled = function() return not db.profile.Enrage.EnableEnrage end,
						},								
						AudioEnrage = {
							order = 33,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							disabled = function() return not db.profile.Enrage.EnableEnrage end,
						},						
						test6 = {
							type = "execute",
							name = L.options["Test"],
							order = 34,
							width = "normal",
							func = function()
								local a = db.profile.Enrage.MessageEnrageColor
								local b = "Interface\\Icons\\Spell_Shadow_UnholyFrenzy"
								local c = db.profile.Enrage.AudioEnrage
								Alerts:NewTMessage(GetSpellInfo(8599),a,b,c)
							end,
							disabled = function() return not db.profile.Enrage.EnableEnrage end,						
						},							
				--	},
				--},	
			},
		}
		msg_args.enrage_group = enrage_group	
		local engage_group = {
			type = "group",
			name = L.options["Engaged"].." & "..L.options["Defeated"],
			order = 11,
			args = {
				blank = genblank(100),
				--[[header = {
					type = "description",
					name = L.options["Here you can configure settings related to bosses engaged and defeated messages"],
					order = 0.1,
					width = "full",
				},--]]
				blank2 = genblank(0.2),
				--[[r1_anchor_header = {
					type = "header",
					name = L.options["Beginning"], 
					order = 1,
				},--]]
				eng_group = {
					type = "group",
					name = L.options["Engaged"],
					inline = true,
					order = 1,
					args = {
						EnableEngage = {
							order = 1,
							type = "toggle",
							name = L.options["Enable"],
							width = "normal",
							get = function(info)
								return db.profile.Engage.EnableEngage
							end,
							set = function(info,v)
								db.profile.Engage.EnableEngage = v
							end,
						},
						EngageTextSize = {
							type = "range",
							name = L.options["Text Size"],
							desc = L.options["Custom Text Size for this ability"],
							order = 2,
							min = 20,
							max = 35,
							step = 1,
							get = function(info)
								return db.profile.Engage.EngageTextSize
							end,
							set = function(info,v)
								db.profile.Engage.EngageTextSize = v
							end,
							disabled = function() return not db.profile.Engage.EnableEngage end,
						},	
						MessageEngageColor = {
							order = 3,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							get = function(info)
								return db.profile.Engage.MessageEngageColor
							end,
							set = function(info,v)
								db.profile.Engage.MessageEngageColor = v
							end,
							disabled = function() return not db.profile.Engage.EnableEngage end,
						},	
						FlashScreenEngage = {
							type = "toggle",
							order = 4,
							width = "normal",
							name = L.options["FlashScreen"],
							get = function(info) return db.profile.Engage.FlashScreenEngage end,
							set = function(info,v)db.profile.Engage.FlashScreenEngage = v end,
							disabled = function() return not db.profile.Engage.EnableEngage end,	
						},	
						AudioEngage = {
							order = 5,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							get = function(info)
								return db.profile.Engage.AudioEngage
							end,
							set = function(info,v)
								db.profile.Engage.AudioEngage = v
							end,
							disabled = function() return not db.profile.Engage.EnableEngage end,
						},	
						test = {
							type = "execute",
							name = L.options["Test"],
							order = 6,
							width = "normal",
							func = function()
								local a = db.profile.Engage.MessageEngageColor
								local b = "INTERFACE\\ENCOUNTERJOURNAL\\UI-EJ-BOSS-Warlord Blackhand.BLP:35:35"
								local c = db.profile.Engage.AudioEngage
								local d = db.profile.Engage.EngageTextSize
								local e = db.profile.Engage.FlashScreenEngage
								Alerts:EngageTest(a,b,c,d,e)
							end,
							disabled = function() return not db.profile.Engage.EnableEngage end,						
						},
						--blank6 = genblank(6.3),
					},
				},
				eng2_group = {
					type = "group",
					name = L.options["Defeated"],
					inline = true,
					order = 2,
					args = {
						EnableDefeated = {
							order = 31,
							type = "toggle",
							name = L.options["Enable"],
							width = "normal",
							get = function(info)
								return db.profile.Engage.EnableDefeated
							end,
							set = function(info,v)
								db.profile.Engage.EnableDefeated = v
							end,
						},
						DefeatedTextSize = {
							type = "range",
							name = L.options["Text Size"],
							desc = L.options["Custom Text Size for this ability"],
							order = 31.1,
							min = 20,
							max = 35,
							step = 1,
							get = function(info)
								return db.profile.Engage.DefeatedTextSize
							end,
							set = function(info,v)
								db.profile.Engage.DefeatedTextSize = v
							end,
							disabled = function() return not db.profile.Engage.EnableDefeated end,
						},	
						--blank6 = genblank(31.5),
						MessageDefeatedColor = {
							order = 32,
							type = "select",
							name = L.options["Messages Color"],
							values = colors,
							get = function(info)
								return db.profile.Engage.MessageDefeatedColor
							end,
							set = function(info,v)
								db.profile.Engage.MessageDefeatedColor = v
							end,
							disabled = function() return not db.profile.Engage.EnableDefeated end,
						},	
						test = {
							type = "execute",
							name = L.options["Test"],
							order = 304,
							width = "normal",
							func = function()
								local a = db.profile.Engage.MessageDefeatedColor
								local b = "INTERFACE\\ENCOUNTERJOURNAL\\UI-EJ-BOSS-Warlord Blackhand.BLP:35:35"
								local c = db.profile.Sounds.VICTORY
								local d = db.profile.Engage.DefeatedTextSize
								local e = db.profile.Engage.FlashScreenDefeated
								Alerts:DefeatedTest(a,b,c,d,e)
							end,
							disabled = function() return not db.profile.Engage.EnableDefeated end,						
						},
						blank7 = genblank(40.5),
						Victory = {
							type = "toggle",
							order = 302,
							width = "normal",
							name = L.options["Victory Sound"],
							desc = L.options["Toggles use of victory sound when a boss is defeated"],
							get = function() return db.profile.Engage.Victory end,
							set = function(info,value) db.profile.Engage.Victory = value end,
							disabled = function() return not db.profile.Engage.EnableEngage end,	
						},
						VictorySound = {
							type = "select",
							name = L.options["Victory Sound"],
							order = 303,
							get = function() return db.profile.Sounds.VICTORY end,
							set = function(info,v) db.profile.Sounds.VICTORY = v end,
							dialogControl = "LSM30_Sound",
							values = addon.SM:HashTable("sound"),
							disabled = function() return not db.profile.Engage.Victory end,
						},
						blank8 = genblank(304.5),
						FlashScreenDefeated = {
							type = "toggle",
							order = 305,
							width = "normal",
							name = L.options["FlashScreen"],
							get = function(info) return db.profile.Engage.FlashScreenDefeated end,
							set = function(info,v)db.profile.Engage.FlashScreenDefeated = v end,
							disabled = function() return not db.profile.Engage.EnableEngage end,	
						},
						--[[header2 = {
							type = "description",
							name = L.options["You can change the victory sound in Sounds-> Sound Labels"],
							order = 305,
							width = "full",
						},--]]
						--[[AudioDefeated = {
							order = 34,
							type = "select",
							name = L.options["Sound"],
							values = GetSounds,
							get = function(info)
								return db.profile.Engage.AudioDefeated
							end,
							set = function(info,v)
								db.profile.Engage.AudioDefeated = v
							end,
							disabled = function() return not db.profile.Engage.EnableDefeated end,
						},	--]]			
					},
				},					
			},
		}
		msg_args.engage_group = engage_group
		
		local InterruptAnnouncer = addon.InterruptAnnouncer
		
		local announce_anchor_group = {
			type = "group",
			name = L.options["Announces"],
			order = 119,
			childGroups = "tab",
			args = {

			},
		}
		alerts_args.announce_anchor_group = announce_anchor_group
		local announce_args = announce_anchor_group.args	
		
		local announce_group = {
			type = "group",
			name = L.options["Announce to raid"],
			order = 1,
			set = function(info,v)
				db.profile.Announces[info[#info]] = v
			end,
			get = function(info)
				return db.profile.Announces[info[#info]]
			end,
			args = {
			--[[	message_anchor_header = {
					type = "header",
					name = L.options["Adjust settings related to the announces"],
					order = 1,
				},--]]
				blank = genblank(1),
				header = {
					type = "description",
					name = L.options["This allows you to change your abilitys announces from including playername or the DXE standart Me. Your healers might prefer playername."],
					order = 2,
					width = "full",
				},
				blank2 = genblank(3),
				blank3 = genblank(4),
				header6_group = {
					type = "group",
					name = L.options["Settings"],					
					order = 203,
					inline = true,
					args = {					
						ReplaceMe = {
							type = "toggle",
							name = function(info)
								local class = select(2,UnitClass("player"))
								local name = UnitName("player")
								return L.options["Replace Me with "]..class_color[class]..name
							end,
							width = "double",
							order = 2,
						},	
					},
				},	
			},
		}
		announce_args.announce_group = announce_group
		
		local spellinterrupt_group = {
			type = "group",
			name = L.options["Spell Interrupt"],
			order = 2,
			set = function(info,v)
				InterruptAnnouncer.db.profile[info[#info]] = v
			end,
			get = function(info)
				return InterruptAnnouncer.db.profile[info[#info]]
			end,
			args = {
				blank = genblank(1),
				header = {
					type = "description",
					name = L.options["This allows you to announce to party or raid when you or other player interrupt a spell"],
					order = 2,
					width = "full",
				},
				blank2 = genblank(3),
				AnnounceInterrupt = {
					type = "toggle",
					name = L.options["Enable"],
					width = "normal",
					order = 3,
				},
				InterruptAnnounceType = {
					type = "select",
					name = L.options["Type"],
					order = 4,
					--values = {AUTOMATIC = L.options["Automatic"], UP = L.options["Up"], DOWN = L.options["Down"]},
					values = {
						["SAY"] = L.options["SAY"],
						["YELL"] = L.options["YELL"],
					},		
				},
				misc_group = {
					type = "group",
					name = L.options["Chat Messages"],
					order = 1311,
					inline = true,
					args = {	
						desc = {
							type = "description",
							name = L.options["This enables the display to raid or party when a interrupt is made from you or from other players"],
							order = 121,
							width = "full",
						},						
						blank = genblank(121.5),
						blank2 = genblank(121.6),							
						InterruptChatMessages = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1215,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.InterruptChatMessages = v
								InterruptAnnouncer.RefreshProfile()
							end,
							--disabled = function() return not InterruptAnnouncer.db.profile.Enabled end,
						},
						blank3 = genblank(1216.5),
						blank4 = genblank(1216.6),
						desc2 = {
							type = "description",
							name = L.options["Here you can choose if the raid or party message is just for your interrupts or from all players"],
							order = 1217,
							width = "full",
						},	
						blank5 = genblank(1217.5),
						blank6 = genblank(1217.6),					
						InterruptOnlyPlayer = {
							type = "toggle",
							name = L.options["Just Yours"],
							order = 1218,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.InterruptOnlyPlayer = v
								if v == true then 
									InterruptAnnouncer.db.profile.InterruptAllPlayers = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.InterruptAllPlayers = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.ChatMessages = false
								end
								InterruptAnnouncer.RefreshProfile()
							end,
							disabled = function() return not InterruptAnnouncer.db.profile.InterruptChatMessages  end,						
						},
						InterruptAllPlayers = {
							type = "toggle",
							name = L.options["All"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.InterruptAllPlayers = v
								if v == true then 
									InterruptAnnouncer.db.profile.InterruptOnlyPlayer = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.InterruptOnlyPlayer = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.InterruptChatMessages = false
								end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.InterruptChatMessages end,
						},
						InterruptLocalMsg = {
							type = "toggle",
							name = L.options["Local Message Only"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.InterruptLocalMsg = v
								--if v == true then 
									--InterruptAnnouncer.db.profile.OnlyPlayer = false
									--InterruptAnnouncer.db.profile.AllPlayers = false
								--else InterruptAnnouncer.db.profile.OnlyPlayer = true end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.InterruptChatMessages end,
						},						
					},
				},				
			},
		}
		announce_args.spellinterrupt_group = spellinterrupt_group	

		local spelldispell_group = {
			type = "group",
			name = L.options["Spell Dispel"],
			order = 3,
			set = function(info,v)
				InterruptAnnouncer.db.profile[info[#info]] = v
			end,
			get = function(info)
				return InterruptAnnouncer.db.profile[info[#info]]
			end,
			args = {
				blank = genblank(1),
				header = {
					type = "description",
					name = L.options["This allows you to announce to party or raid when you or other player dispel a spell"],
					order = 2,
					width = "full",
				},
				blank2 = genblank(3),
				AnnounceDispelled = {
					type = "toggle",
					name = L.options["Enable"],
					width = "normal",
					order = 3,
				},
				DispellAnnounceType = {
					type = "select",
					name = L.options["Type"],
					order = 4,
					values = {
						["SAY"] = L.options["SAY"],
						["YELL"] = L.options["YELL"],
					},		
				},
				misc_group = {
					type = "group",
					name = L.options["Chat Messages"],
					order = 1311,
					inline = true,
					args = {	
						desc = {
							type = "description",
							name = L.options["This enables the display to raid or party when a dispel is made from you or from other players"],
							order = 121,
							width = "full",
						},						
						blank = genblank(121.5),
						blank2 = genblank(121.6),							
						DispellChatMessages = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1215,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.DispellChatMessages = v
								InterruptAnnouncer.RefreshProfile()
							end,
							--disabled = function() return not InterruptAnnouncer.db.profile.Enabled end,
						},
						blank3 = genblank(1216.5),
						blank4 = genblank(1216.6),
						desc2 = {
							type = "description",
							name = L.options["Here you can choose if the raid or party message is just for your dispel or from all players"],
							order = 1217,
							width = "full",
						},	
						blank5 = genblank(1217.5),
						blank6 = genblank(1217.6),					
						DispellOnlyPlayer = {
							type = "toggle",
							name = L.options["Just Yours"],
							order = 1218,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.DispellOnlyPlayer = v
								if v == true then 
									InterruptAnnouncer.db.profile.DispellAllPlayers = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.DispellAllPlayers = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.ChatMessages = false
								end
								InterruptAnnouncer.RefreshProfile()
							end,
							disabled = function() return not InterruptAnnouncer.db.profile.DispellChatMessages  end,						
						},
						DispellAllPlayers = {
							type = "toggle",
							name = L.options["All"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.DispellAllPlayers = v
								if v == true then 
									InterruptAnnouncer.db.profile.DispellOnlyPlayer = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.DispellOnlyPlayer = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.DispellChatMessages = false
								end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.DispellChatMessages end,
						},
						DispellLocalMsg = {
							type = "toggle",
							name = L.options["Local Message Only"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.DispellLocalMsg = v
								--if v == true then 
									--InterruptAnnouncer.db.profile.OnlyPlayer = false
									--InterruptAnnouncer.db.profile.AllPlayers = false
								--else InterruptAnnouncer.db.profile.OnlyPlayer = true end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.DispellChatMessages end,
						},						
					},
				},				
			},
		}
		announce_args.spelldispell_group = spelldispell_group	

		local spellstolen_group = {
			type = "group",
			name = L.options["Spell Steal"],
			order = 4,
			set = function(info,v)
				InterruptAnnouncer.db.profile[info[#info]] = v
			end,
			get = function(info)
				return InterruptAnnouncer.db.profile[info[#info]]
			end,
			args = {
				blank = genblank(1),
				header = {
					type = "description",
					name = L.options["This allows you to announce to party or raid when you or other player steals a spell"],
					order = 2,
					width = "full",
				},
				blank2 = genblank(3),
				AnnounceSpellSteal = {
					type = "toggle",
					name = L.options["Enable"],
					width = "normal",
					order = 3,
				},
				SpellStealAnnounceType = {
					type = "select",
					name = L.options["Type"],
					order = 4,
					--values = {AUTOMATIC = L.options["Automatic"], UP = L.options["Up"], DOWN = L.options["Down"]},
					values = {
						["SAY"] = L.options["SAY"],
						["YELL"] = L.options["YELL"],
					},		
				},
				misc_group = {
					type = "group",
					name = L.options["Chat Messages"],
					order = 1311,
					inline = true,
					args = {	
						desc = {
							type = "description",
							name = L.options["This enables the display to raid or party when a steal is made from you or from other players"],
							order = 121,
							width = "full",
						},						
						blank = genblank(121.5),
						blank2 = genblank(121.6),							
						SpellStealChatMessages = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1215,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.SpellStealChatMessages = v
								InterruptAnnouncer.RefreshProfile()
							end,
							--disabled = function() return not InterruptAnnouncer.db.profile.Enabled end,
						},
						blank3 = genblank(1216.5),
						blank4 = genblank(1216.6),
						desc2 = {
							type = "description",
							name = L.options["Here you can choose if the raid or party message is just for your steals or from all players"],
							order = 1217,
							width = "full",
						},	
						blank5 = genblank(1217.5),
						blank6 = genblank(1217.6),					
						SpellStealOnlyPlayer = {
							type = "toggle",
							name = L.options["Just Yours"],
							order = 1218,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.SpellStealOnlyPlayer = v
								if v == true then 
									InterruptAnnouncer.db.profile.SpellStealAllPlayers = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.SpellStealAllPlayers = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.ChatMessages = false
								end
								InterruptAnnouncer.RefreshProfile()
							end,
							disabled = function() return not InterruptAnnouncer.db.profile.SpellStealChatMessages  end,						
						},
						SpellStealAllPlayers = {
							type = "toggle",
							name = L.options["All"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.SpellStealAllPlayers = v
								if v == true then 
									InterruptAnnouncer.db.profile.SpellStealOnlyPlayer = false
									--InterruptAnnouncer.db.profile.PrintMsg = false
								else 
									InterruptAnnouncer.db.profile.SpellStealOnlyPlayer = true 
									--InterruptAnnouncer.db.profile.PrintMsg = true 
									--InterruptAnnouncer.db.profile.DispellChatMessages = false
								end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.SpellStealChatMessages end,
						},
						SpellStealLocalMsg = {
							type = "toggle",
							name = L.options["Local Message Only"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								InterruptAnnouncer.db.profile.SpellStealLocalMsg = v
								--if v == true then 
									--InterruptAnnouncer.db.profile.OnlyPlayer = false
									--InterruptAnnouncer.db.profile.AllPlayers = false
								--else InterruptAnnouncer.db.profile.OnlyPlayer = true end									
								InterruptAnnouncer.RefreshProfile()
							end,	
							disabled = function() return not InterruptAnnouncer.db.profile.SpellStealChatMessages end,
						},						
					},
				},				
			},
		}
		announce_args.spellstolen_group = spellstolen_group		
		
		local Arrows = addon.Arrows
		local arrows_group = {
			name = L.options["Arrows"],
			type = "group",
			order = 120,
			get = function(info) return Arrows.db.profile[info[#info]] end,
			set = function(info,v) Arrows.db.profile[info[#info]] = v; Arrows:RefreshArrows() end,
			args = {
				Enable = {
					type = "toggle",
					name = L.options["Enable"],
					desc = L.options["Enable the use of directional arrows"],
					order = 1,
				},
				blank2 = genblank(1.6),
				enable_group = {
					type = "group",
					order = 2,
					name = "Settings",
					inline = true,
					disabled = function() return not Arrows.db.profile.Enable end,
					args = {
						TestArrows = {
							name = L.options["Test Arrows"],
							type = "execute",
							order = 1,
							desc = L.options["Displays all arrows and then rotates them for ten seconds"],
							func = function()
								for k,arrow in ipairs(addon.Arrows.frames) do
									arrow:Test()
								end
							end,
						},
						Scale = {
							name = L.options["Scale"],
							desc = L.options["Adjust the scale of arrows"],
							type = "range",
							min = 0.3,
							max = 2,
							step = 0.1
						},
					},
				},
			},
		}

		alerts_args.arrows_group = arrows_group

		local RaidIcons = addon.RaidIcons

		local raidicons_group = {
			type = "group",
			name = L.options["Raid Icons"],
			order = 121,
			get = function(info) return RaidIcons.db.profile[tonumber(info[#info])] end,
			set = function(info,v) RaidIcons.db.profile[tonumber(info[#info])] = v end,
			args = {
				Enabled = {
					type = "toggle",
					get = function(info) return RaidIcons.db.profile.Enabled end,
					set = function(info,v) RaidIcons.db.profile.Enabled = v end,
					name = L.options["Enable"],
					desc = L.options["Enable raid icon marking. If this is disabled then you will not mark targets even if you have raid assist and raid icons enabled in encounters"],
					order = 0.1,
				},
			}
		}

		do
			local raidicons_args = raidicons_group.args

			local desc = {
				type = "description",
				name = L.options["Most encounters only use |cffffd200Icon 1|r and |cffffd200Icon 2|r. Additional icons are used for abilities that require multi-marking (e.g. Anub'arak's Penetrating Cold). If you change an icon, make sure all icons are different from one another"],
				order = 0.5,
			}

			raidicons_args.desc = desc

			local dropdown = {
				type = "select",
				name = function(info) return format(L.options["Icon %s"],info[#info]) end,
				order = function(info) return tonumber(info[#info]) end,
				width = "double",
				values = {
					"1. "..L.options["Star"],
					"2. "..L.options["Circle"],
					"3. "..L.options["Diamond"],
					"4. "..L.options["Triangle"],
					"5. "..L.options["Moon"],
					"6. "..L.options["Square"],
					"7. "..L.options["Cross"],
					"8. "..L.options["Skull"],
				},
			}

			for i=1,8 do raidicons_args[tostring(i)] = dropdown end
		end

		alerts_args.raidicons_group = raidicons_group

	end

	---------------------------------------------
	-- DISTRIBUTOR
	---------------------------------------------

	do
		local list,names = {},{}
		local ListSelect,PlayerSelect
		local Distributor = addon.Distributor
		local dist_group = {
			type = "group",
			name = L.options["Distributor"],
			order = 350,
			get = function(info) return Distributor.db.profile[info[#info]] end,
			set = function(info,v) Distributor.db.profile[info[#info]] = v end,
			args = {
				AutoAccept = {
					type = "toggle",
					name = L.options["Auto accept"],
					desc = L.options["Automatically accepts encounters sent by players"],
					order = 50,
				},
				first_desc = {
					type = "description",
					order = 75,
					name = format(L.options["You can send encounters to the entire raid or to a player. You can check versions by typing |cffffd200/dxe %s|r or by opening the version checker from the pane"],L.options["version"]),
				},
				raid_desc = {
					type = "description",
					order = 90,
					name = "\n"..L.options["If you want to send an encounter to the raid, select an encounter, and then press '|cffffd200Send to raid|r'"],
				},
				ListSelect = {
					type = "select",
					order = 100,
					name = L.options["Select an encounter"],
					get = function() return ListSelect end,
					set = function(info,value) ListSelect = value end,
					values = function()
						wipe(list)
						for k in addon:IterateEDB() do list[k] = addon.EDB[k].name end
						return list
					end,
				},
				DistributeToRaid = {
					type = "execute",
					name = L.options["Send to raid"],
					order = 200,
					func = function() Distributor:Distribute(ListSelect) end,
					disabled = function() return GetNumGroupMembers() == 0 or not ListSelect  end,
				},
				player_desc = {
					type = "description",
					order = 250,
					name = "\n\n"..L.options["If you want to send an encounter to a player, select an encounter, select a player, and then press '|cffffd200Send to player|r'"],
				},
				PlayerSelect = {
					type = "select",
					order = 300,
					name = L.options["Select a player"],
					get = function() return PlayerSelect end,
					set = function(info,value) PlayerSelect = value end,
					values = function()
						wipe(names)
						for name in pairs(addon.Roster.name_to_unit) do
							if name ~= addon.PNAME then names[name] = name end
						end
						return names
					end,
					disabled = function() return  not ListSelect end, --GetNumSubgroupMembers() == 0
				},
				DistributeToPlayer = {
					type = "execute",
					order = 400,
					name = L.options["Send to player"],
					func = function() Distributor:Distribute(ListSelect, PlayerSelect) end,
					disabled = function() return not PlayerSelect end,
				},
			},
		}

		opts_args.dist_group = dist_group
	end

	---------------------------------------------
	-- WINDOWS
	---------------------------------------------

	do
		windows_group = {
			type = "group",
			name = L.options["Windows"],
			order = 290,
			childGroups = "tab",
			args = {
				TitleBarColor = {
					order = 100,
					type = "color",
					set = function(info,v,v2,v3,v4)
						local t = db.profile.Windows.TitleBarColor
						t[1],t[2],t[3],t[4] = v,v2,v3,v4
						addon:UpdateWindowSettings()
					end,
					get = function(info) return unpack(db.profile.Windows.TitleBarColor) end,
					name = L.options["Title Bar Color"],
					desc = L.options["Title bar color used throughout the addon"],
					hasAlpha = true,
				},
				Proxtype = {
					type = "select",
					order = 110,
					get = function(info) return db.profile.Windows.Proxtype end,
					set = function(info,v)
						db.profile.Windows.Proxtype = v
						addon:UpdateProximityProfile()
						addon:ShowProximity()
					end,
					name = L.options["Proximity Display"],
					desc = L.options["The type of the proximity window, radar or text-based"],
					values = {
						RADAR = L.options["Radar"],
						TEXT = L.options["Text-based"],
					},
				},
			},
		}

		opts_args.windows_group = windows_group
		local windows_args = windows_group.args

		local alternatepower_group = {
			type = "group",
			name = L.options["AlternatePower"],
			order = 149,
			get = function(info) return db.profile.AlternatePower[info[#info]] end,
			set = function(info,v) db.profile.AlternatePower[info[#info]] = v; addon:UpdateAlternatePowerSettings() end,
			args = {
				header_desc = {
					type = "description",
					order = 1,
					name = L.options["The alternate power window tracks the 'alternate power' of players for certain fights ie- Cho'gall and Atremedes"].."\n",
				},
				toggles_group = {
					type = "group",
					name = L.options["Toggles"],
					order = 2,
					inline = true,
					args = {
						AutoPopup = {
							type = "toggle",
							order = 50,
							width = "full",
							name = L.options["Auto Popup"],
							desc = L.options["Automatically show the alternate power window if the option is enabled in an encounter (Encounters > ... > Windows > AlternatePower)"],
						},
						AutoHide = {
							type = "toggle",
							order = 60,
							width = "full",
							name = L.options["Auto Hide"],
							desc = L.options["Automatically hide the alternate power window when an encounter is defeated"],
						},
						Invert = {
							type = "toggle",
							order = 75,
							width = "full",
							name = L.options["Invert Bars"],
							desc = L.options["Inverts all power bars"],
						},
						Dummy = {
							type = "toggle",
							order = 80,
							width = "full",
							name = L.options["Dummy Bars"],
							desc = L.options["Displays dummy bars that can be useful for configuration"],
						},
					},
				},
				general_group = {
					type = "group",
					name = L.options["General"],
					order = 3,
					inline = true,
					args = {
						Threshold = {
							type = "range",
							order = 99,
							name = L.options["Power Threshold"],
							desc = L.options["A minimum amount of alternate power a player must have to be displayed."],
							min = 1,
							max = 99,
							step = 1,
						},
						Delay = {
							type = "range",
							order = 200,
							name = L.options["Delay"],
							desc = L.options["The alternatePower window refresh rate (seconds). Increase to improve performance. |cff99ff330|r refreshes every frame"],
							min = 0,
							max = 1,
							step = 0.05,
						},
						BarAlpha = {
							type = "range",
							order = 250,
							name = L.options["Bar Alpha"],
							desc = L.options["Adjust the transparency of range bars"],
							min = 0.1,
							max = 1,
							step = 0.1,
						},
						Rows = {
							type = "range",
							order = 260,
							name = L.options["Rows"],
							desc = L.options["The number of bars to show"],
							min = 1,
							max = 15,
							step = 1,
						},
						IconPosition = {
							type = "select",
							order = 270,
							name = L.options["Icon Position"],
							desc = L.options["The position of the class icon"],
							values = {
								LEFT = L.options["Left"],
								RIGHT = L.options["Right"],
							},
						},
					},
				},
				font_group = {
					type = "group",
					name = L.options["Text"],
					order = 4,
					inline = true,
					args = {
						NameFontSize = {
							order = 1,
							type = "range",
							name = L.options["Name Font Size"],
							desc = L.options["Select a font size used on name text"],
							min = 7,
							max = 30,
							step = 1,
						},
						NameOffset = {
							order = 2,
							type = "range",
							name = L.options["Name Horizontal Offset"],
							desc = L.options["The horizontal position of name text"],
							min = -175,
							max = 175,
							step = 1,
						},
						NameAlignment = {
							order = 2.3,
							type = "select",
							name = L.options["Name Alignment"],
							desc = L.options["The text alignment of the name text"],
							values = {
								LEFT = L.options["Left"],
								CENTER = L.options["Center"],
								RIGHT = L.options["Right"],
							},
						},
						blank = genblank(2.5),
						TimeFontSize = {
							order = 3,
							type = "range",
							name = L.options["Power Font Size"],
							desc = L.options["Select a font size used on power text"],
							min = 7,
							max = 30,
							step = 1,
						},
						TimeOffset = {
							order = 4,
							type = "range",
							name = L.options["Power Horizontal Offset"],
							desc = L.options["The horizontal position of power text"],
							min = -220,
							max = 175,
							step = 1,
						},
					},
				},
				ClassFilter = {
					type = "multiselect",
					order = 5,
					name = L.options["Class Filter"],
					get = function(info,v) return db.profile.AlternatePower.ClassFilter[v] end,
					set = function(info,v,v2) db.profile.AlternatePower.ClassFilter[v] = v2 end,
					values = LOCALIZED_CLASS_NAMES_MALE,
				},
			},
		}

		windows_args.alternatepower_group = alternatepower_group

		local proximity_group = {
			type = "group",
			name = L.options["Proximity"],
			order = 150,
			get = function(info) return db.profile.Proximity[info[#info]] end,
			set = function(info,v) db.profile.Proximity[info[#info]] = v; addon:UpdateProximityProfile(); addon:UpdateWindowSettings() end,
			args = {
				header_desc = {
					type = "description",
					order = 1,
					name = L.options["The proximity window uses map coordinates of players to calculate distances. This relies on knowing the dimensions, in game yards, of each map. If the dimension of a map is not known, it will default to the closest range rounded up to 10, 11, or 18 game yards"].."\n",
				},
				toggles_group = {
					type = "group",
					name = L.options["Toggles"],
					order = 2,
					inline = true,
					args = {
						AutoPopup = {
							type = "toggle",
							order = 50,
							width = "full",
							name = L.options["Auto Popup"],
							desc = L.options["Automatically show the proximity window if the option is enabled in an encounter (Encounters > ... > Windows > Proximity)"],
						},
						AutoHide = {
							type = "toggle",
							order = 60,
							width = "full",
							name = L.options["Auto Hide"],
							desc = L.options["Automatically hide the proximity window when an encounter is defeated"],
						},
						Sound = {
							type = "toggle",
							order = 70,
							width = "full",
							name = L.options["Proximity Sound"],
							--desc = L.options["Automatically hide the proximity window when an encounter is defeated"],
						},						
						--[[Invert = {
							type = "toggle",
							order = 75,
							width = "full",
							name = L.options["Invert Bars"],
							desc = L.options["Inverts all range bars"],
						},--]]
						Dummy = {
							type = "toggle",
							order = 80,
							width = "full",
							name = L.options["Dummy Bars/Dots"],
							desc = L.options["Displays dummy bars or dots on the radar that can be useful for configuration"],
						},
					},
				},
				general_group = {
					type = "group",
					name = L.options["General"],
					order = 3,
					inline = true,
					args = {
						Range = {
							type = "range",
							order = 100,
							name = L.options["Range"],
							desc = L.options["The distance (game yards) a player has to be within to appear in the proximity window"],
							min = 2,
							max = 18,
							step = 1,
						},
						Delay = {
							type = "range",
							order = 200,
							name = L.options["Delay"],
							desc = L.options["The proximity window refresh rate (seconds). Increase to improve performance. |cff99ff330|r refreshes every frame"],
							min = 0,
							max = 1,
							step = 0.05,
						},
						BarAlpha = {
							type = "range",
							order = 250,
							name = L.options["Bar Alpha"],
							desc = L.options["Adjust the transparency of range bars"],
							min = 0.1,
							max = 1,
							step = 0.1,
						},
						Rows = {
							type = "range",
							order = 260,
							name = L.options["Rows"],
							desc = L.options["The number of bars to show"],
							min = 1,
							max = 15,
							step = 1,
						},
						IconPosition = {
							type = "select",
							order = 270,
							name = L.options["Icon Position"],
							desc = L.options["The position of the class icon"],
							values = {
								LEFT = L.options["Left"],
								RIGHT = L.options["Right"],
							},
						},
						ProximitySound = {
							type = "select",
							name = L.options["Proximity Sound"],
							order = 280,
							get = function() return db.profile.Sounds.PROXIMITY end,
							set = function(info,v) db.profile.Sounds.PROXIMITY = v end,
							dialogControl = "LSM30_Sound",
							values = addon.SM:HashTable("sound"),
							disabled = function() return not db.profile.Proximity.Sound end,
						},						
					},
				},
				radar_group = {
					type = "group",
					name = L.options["Radar"],
					order = 3.5,
					inline = true,
					args = {
						DotSize = {
							type = "range",
							order = 100,
							name = L.options["Dot Size"],
							desc = L.options["The size of the dots and icons in the radar window"],
							min = 5,
							max = 20,
							step = 1,
						},
						RaidIcons = {
							type = "select",
							order = 200,
							name = L.options["Raidicon Display"],
							desc = L.options["Set the visibilty of raidicons: None, Above the dots, Replace the dots"],
							values = {
								NONE = L.options["None"],
								ABOVE = L.options["Above"],
								REPLACE = L.options["Replace"],
							},
						},
					},
				},
				font_group = {
					type = "group",
					name = L.options["Text"],
					order = 4,
					inline = true,
					args = {
						NameFontSize = {
							order = 1,
							type = "range",
							name = L.options["Name Font Size"],
							desc = L.options["Select a font size used on name text"],
							min = 7,
							max = 30,
							step = 1,
						},
						NameOffset = {
							order = 2,
							type = "range",
							name = L.options["Name Horizontal Offset"],
							desc = L.options["The horizontal position of name text"],
							min = -175,
							max = 175,
							step = 1,
						},
						NameAlignment = {
							order = 2.3,
							type = "select",
							name = L.options["Name Alignment"],
							desc = L.options["The text alignment of the name text"],
							values = {
								LEFT = L.options["Left"],
								CENTER = L.options["Center"],
								RIGHT = L.options["Right"],
							},
						},
						blank = genblank(2.5),
						TimeFontSize = {
							order = 3,
							type = "range",
							name = L.options["Time Font Size"],
							desc = L.options["Select a font size used on time text"],
							min = 7,
							max = 30,
							step = 1,
						},
						TimeOffset = {
							order = 4,
							type = "range",
							name = L.options["Time Horizontal Offset"],
							desc = L.options["The horizontal position of time text"],
							min = -220,
							max = 175,
							step = 1,
						},
					},
				},
				ClassFilter = {
					type = "multiselect",
					order = 5,
					name = L.options["Class Filter"],
					get = function(info,v) return db.profile.Proximity.ClassFilter[v] end,
					set = function(info,v,v2) db.profile.Proximity.ClassFilter[v] = v2 end,
					values = LOCALIZED_CLASS_NAMES_MALE,
				},
			},
		}

		windows_args.proximity_group = proximity_group
		
		local anchor_group = {
			type = "group",
			name = L.options["Pixel Perfect"],
			order = 160,
			get = function(info) return db.profile.Proximity[info[#info]] end,
			set = function(info,v) db.profile.Proximity[info[#info]] = v; addon:UpdateProximityProfile() end,
			args = {
				alternatep_group = {
					type = "group",
					name = L.options["AlternatePower"],
					order = 1,
					inline = true,
					args = {
						blan = genblank(1.1),
					--	blan2 = genblank(1.2),
						--[[PixelPerfect_header = {
							type = "header",
							name = L.options["Pixel Perfect"],
							order = 2,
						},--]]					
						blank = genblank(3),
						blank2 = genblank(3.1),
						PaneAnchor_x = {
							type = "range",
							name = L.options["X Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 4,
							width = "full",
							get = function(info) 
								local pos = db.profile.Positions["DXEWindowAlternatePower"]
								if pos then
									return pos.xOfs
								else
									return 179.0000387755648
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowAlternatePower"].xOfs = v
								addon:LoadPosition("DXEWindowAlternatePower")
							end,							
						},
						PaneAnchor_y = {
							type = "range",
							name = L.options["Y Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 5,
							width = "full",
							get = function(info)
								local pos = db.profile.Positions["DXEWindowAlternatePower"]
								if pos then
									return pos.yOfs
								else
									return -158.9998021557163
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowAlternatePower"].yOfs = v
								addon:LoadPosition("DXEWindowAlternatePower")
							end,
						},
					},
				},
				radar_group = {
					type = "group",
					name = L.options["Radar"],
					order = 2,
					inline = true,
					args = {
						--blan1 = genblank(6.1),
						blan12 = genblank(6.2),
						--[[Radar_header = {
							type = "header",
							name = L.options["Pixel Perfect"],
							order = 7,
						},	--]]
						blank1 = genblank(8),
						blank12 = genblank(8.1),
						RadarAnchor_x = {
							type = "range",
							name = L.options["X Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 9,
							width = "full",
							get = function(info) 
								local pos = db.profile.Positions["DXEWindowRadar"]
								if pos then
									return pos.xOfs
								else
									return 179.0000387755648
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowRadar"].xOfs = v
								addon:LoadPosition("DXEWindowRadar")
							end,							
						},
						RadarAnchor_y = {
							type = "range",
							name = L.options["Y Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 10,
							width = "full",
							get = function(info)
								local pos = db.profile.Positions["DXEWindowRadar"]
								if pos then
									return pos.yOfs
								else
									return -158.9998021557163
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowRadar"].yOfs = v
								addon:LoadPosition("DXEWindowRadar")
							end,
						},
					},
				},
				proximity_group = {
					type = "group",
					name = L.options["Proximity"],
					order = 3,
					inline = true,
					args = {
						--blan2 = genblank(11.1),
						blan22 = genblank(11.2),
						--[[Proximity_header = {
							type = "header",
							name = L.options["Pixel Perfect"],
							order = 12,
						},	--]]
						blank2 = genblank(13),
						blank22 = genblank(13.1),
						ProximityAnchor_x = {
							type = "range",
							name = L.options["X Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 14,
							width = "full",
							get = function(info) 
								local pos = db.profile.Positions["DXEWindowProximity"]
								if pos then
									return pos.xOfs
								else
									return 179.0000387755648
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowProximity"].xOfs = v
								addon:LoadPosition("DXEWindowProximity")
							end,							
						},
						ProximityAnchor_y = {
							type = "range",
							name = L.options["Y Position"],
							min = -2048,
							max = 2048,
							step = 1,
							order = 15,
							width = "full",
							get = function(info)
								local pos = db.profile.Positions["DXEWindowProximity"]
								if pos then
									return pos.yOfs
								else
									return -158.9998021557163
								end
							end,
							set = function(info,v)
								db.profile.Positions["DXEWindowProximity"].yOfs = v
								addon:LoadPosition("DXEWindowProximity")
							end,
						},
					},
				},				
			},
		}

		windows_args.anchor_group = anchor_group		
	end

	---------------------------------------------
	-- SOUNDS
	---------------------------------------------

	do
		local sounds = {}
		-- Same function at the very top of this function, without adding None
		local function GetSounds()
			table.wipe(sounds)
			for id,name in pairs(db.profile.Sounds) do
				if id:find("^ALERT") or id == "VICTORY" or id == "PROXIMITY" then sounds[id] = id end
			end
			for id,name in pairs(db.profile.CustomSounds) do
				sounds[id] = id
			end
			return sounds
		end
		
		local Alerts = addon.Alerts
		local function SetNoRefresh(info,v,v2,v3,v4)
			local var = info[#info]
			if var:find("Color") then
				local c = Alerts.db.profile[var]
				c[1],c[2],c[3],c[4] = v,v2,v3,v4
			else Alerts.db.profile[var] = v end
		end

		local label = "ALERT1"		
		local add_sound_label = ""
		local remove_sound_label = ""
		local remove_list = {}
		local SelectedEncounter
		local EncounterList = {}

		local sound_defaults = addon.defaults.profile.Sounds

		local sounds_group = {
			type = "group",
			name = L.options["Sounds"],
			order = 295,
			childGroups = "tab",
			args = {

			},
		}
		opts_args.sounds_group = sounds_group
		local sounds_args = sounds_group.args

		local soundlabel_group = {
					type = "group",
					name = L.options["Sound Labels"],
					--inline = true,
					order = 1,
					args = {
						insound_group = {
							type = "group",
							name = L.options["Sound Labels"],
							order = 1,
							inline = true,
							args = {
								desc1 = {
									type = "description",
									name = L.options["You can change the sound labels (ALERT1, ALERT2, etc.) to any sound file in SharedMedia. First, select one to change"].."\n",
									order = 1,
								},
								identifier = {
									type = "select",
									name = L.options["Sound Label"],
									order = 2,
									get = function() return label end,
									set = function(info,v) label = v end,
									values = GetSounds,
								},
								reset = {
									type = "execute",
									name = L.options["Reset"],
									desc = L.options["Sets the selected sound label back to its default value"],
									func = function()
										if sound_defaults[label] then
											db.profile.Sounds[label] = sound_defaults[label]
										else
											db.profile.CustomSounds[label] = "None"
										end
									end,
									order = 3
								},
								desc2 = {
									type = "description",
									name = "\n"..L.options["Now change the sound to what you want. Sounds can be tested by clicking on the speaker icons within the dropdown"].."\n",
									order = 4,
								},
								choose = {
									name = function() return format(L.options["Sound File for %s"],label) end,
									order = 5,
									type = "select",
									get = function(info) return sound_defaults[label] and db.profile.Sounds[label] or db.profile.CustomSounds[label] end,
									set = function(info,v)
										if sound_defaults[label] then
											db.profile.Sounds[label] = v
										else
											db.profile.CustomSounds[label] = v
										end
									end,
									dialogControl = "LSM30_Sound",
									values = addon.SM:HashTable("sound"),
								},
							},
						},
						insound2_group = {
							type = "group",
							name = L.options["Add Sound Label"],
							order = 2,
							inline = true,
							args = {
								add_desc = {
									type = "description",
									name = L.options["You can add your own sound label. Each sound label is associated with a certain sound file. Consult SharedMedia's documentation if you would like to add your own sound file. After adding a sound label, it will appear in the Sound Label list. You can then select a sound file to associate with it. Subsequently, the sound label will be available in the encounter options"],
									order = 7,
								},
								sound_label_input = {
									type = "input",
									name = L.options["Label name"],
									order = 8,
									get = function(info) return add_sound_label end,
									set = function(info,v) add_sound_label = v end,
								},
								sound_label_add = {
									type = "execute",
									name = L.options["Add"],
									order = 9,
									func = function()
										db.profile.CustomSounds[add_sound_label] = "None"
										label = add_sound_label
										add_sound_label = ""
									end,
									disabled = function() return add_sound_label == "" end
								},
								remove_desc = {
									type = "description",
									name = "\n"..L.options["You can remove custom sounds labels. Select a sound label from the dropdown and then click remove"],
									order = 9.5,
								},
								sound_label_list = {
									type = "select",
									order = 10,
									name = L.options["Custom Sound Labels"],
									get = function()
										return remove_sound_label
									end,
									set = function(info,v)
										remove_sound_label = v
									end,
									values = function()
										table.wipe(remove_list)
										for k,v in pairs(db.profile.CustomSounds) do remove_list[k] = k end
										return remove_list
									end,
								},
								sound_label_remove = {
									type = "execute",
									name = L.options["Remove"],
									order = 11,
									func = function()
										db.profile.CustomSounds[remove_sound_label] = nil
										if label == remove_sound_label then
											label = "ALERT1"
										end
										remove_sound_label = ""
									end,
									disabled = function() return remove_sound_label == "" end,
								},
							},
						},
						insound3_group = {
							type = "group",
							name = L.options["Set audio channel used by DXE to play alert sounds"],
							order = 3,
							inline = true,
							args = {
								MasterSound = {
									type = "toggle",
									order = 1,
									width = "double",
									name = L.options["Master Sound Channel"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.MasterSound end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = false end
										Alerts.db.profile.MasterSound = value
										Alerts.db.profile.SFXSound = false
										Alerts.db.profile.MusicSound = false
										Alerts.db.profile.AmbienceSound = false
										Alerts.db.profile.DialogSound = false
									end,
								},
								SFXSound = {
									type = "toggle",
									order = 2,
									width = "double",
									name = L.options["Sound Effects (SFX) Audio Channel"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.SFXSound end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = false end
										Alerts.db.profile.SFXSound = value
										Alerts.db.profile.MasterSound = false
										Alerts.db.profile.MusicSound = false
										Alerts.db.profile.AmbienceSound = false
										Alerts.db.profile.DialogSound = false								
									end,
								},	
								MusicSound = {
									type = "toggle",
									order = 3,
									width = "double",
									name = L.options["Music Audio Channel"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.MusicSound end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = false end
										Alerts.db.profile.MusicSound = value
										Alerts.db.profile.MasterSound = false
										Alerts.db.profile.SFXSound = false
										Alerts.db.profile.AmbienceSound = false
										Alerts.db.profile.DialogSound = false
									end,
								},
								AmbienceSound = {
									type = "toggle",
									order = 4,
									width = "double",
									name = L.options["Ambience Audio Channel"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.AmbienceSound end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = false end
										Alerts.db.profile.AmbienceSound = value
										Alerts.db.profile.MasterSound = false
										Alerts.db.profile.SFXSound = false
										Alerts.db.profile.MusicSound = false
										Alerts.db.profile.DialogSound = false
									end,
								},									
								DialogSound = {
									type = "toggle",
									order = 5,
									width = "double",
									name = L.options["Dialog Audio Channel"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.DialogSound end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = false end
										Alerts.db.profile.DialogSound = value
										Alerts.db.profile.MasterSound = false
										Alerts.db.profile.SFXSound = false
										Alerts.db.profile.MusicSound = false
										Alerts.db.profile.AmbienceSound = false
									end,
								},
								NormalSound = {
									type = "toggle",
									order = 6,
									width = "double",
									name = L.options["Normal Audio Channel (lower sound)"],
									--desc = L.options["Toggles use of Master sound channel for alert and arrow sounds"],
									get = function() return Alerts.db.profile.NormalSound end,
									set = function(info,value) Alerts.db.profile.NormalSound = value end,
									set = function(info,value)
										if value == false then Alerts.db.profile.NormalSound = true
										else Alerts.db.profile.NormalSound = value end
										Alerts.db.profile.MasterSound = false
										Alerts.db.profile.SFXSound = false
										Alerts.db.profile.MusicSound = false
										Alerts.db.profile.AmbienceSound = false								
									end,
								},									
							},
						},					
					},
			}
			sounds_args.soundlabel_group = soundlabel_group
			
			local countdownvoice_group = {
					type = "group",
					name = L.options["Countdown voice"],
					order = 2,
					args = {
						blank = genblank(0.5),
						--[[add_desc = {
							type = "description",
							name = L.options["Here you can select a diferent sound countdown voice"],
							order = 1,
						},--]]
						blank2 = genblank(1.5),
						incountdownvoice_group = {
							type = "group",
							name = L.options["Countdown voice"],
							order = 400,
							inline = true,
							args = {
								VoiceSound = {
									order = 3,
									type = "select",
									name = L.options["Countdown voice"],
									--desc = L.options[""],
									values = {
										US_Female = L.options["English: Female"], 
										US_Male = L.options["English: Male"],
										CN_Male = L.options["Chinese: Male"],
										CN_Female = L.options["Chinese: Female"],
										RU_Female = L.options["Russian: Female"],
										RU_Male = L.options["Russian: Male"],
									},
									get = function(info)
										return db.profile.Sounds.VoiceSound
									end,
									set = function(info,v)
										db.profile.Sounds.VoiceSound = v
									end,
								},						
								test = {
									type = "execute",
									name = L.options["Test"],
									order = 4,
									func = function()
										addon.Alerts:AudioCountdownTest(L.options["Countdown voice"])
									end,
								},
							},
						},
					},
			}
			sounds_args.countdownvoice_group = countdownvoice_group
			
			local soundsin_group = {
					type = "group",
					name = L.options["Mute all"],
					order = 3,
					set = SetNoRefresh,
					args = {
						DisableSounds = {
							order = 100,
							type = "toggle",
							name = L.options["Mute all"],
							desc = L.options["Silences all alert sounds"],
							get = function() return Alerts.db.profile.DisableSounds end,
							set = function(info,value) Alerts.db.profile.DisableSounds = value end,
						},
						DisableAll = {
							order = 200,
							type = "execute",
							name = L.options["Set all to None"],
							desc = L.options["Sets every alert's sound to None. This affects currently loaded encounters"],
							func = function()
								for key,tbl in pairs(db.profile.Encounters) do
									for var,stgs in pairs(tbl) do
										if stgs.sound then stgs.sound = "None" end
									end
								end
							end,
							get = function() return Alerts.db.profile.DisableAll end,
							set = function(info,value) Alerts.db.profile.DisableAll = value end,
							confirm = true,							
						},
						reset = {
							type = "execute",
							name = L.options["Reset"],
							order = 250,
							func = function()
								for key,tbl in pairs(db.profile.Encounters) do
								  for var,stgs in pairs(tbl) do
									if stgs.sound then
										stgs.sound = addon.defaults.profile.Encounters[key][var].sound
									end
								  end
								end
							end,
						},											
						curr_enc_group = {
							type = "group",
							name = L.options["Change encounter"],
							order = 400,
							inline = true,
							args = {
								SelectedEncounter = {
									order = 100,
									type = "select",
									name = L.options["Select encounter"],
									desc = L.options["The encounter to change"],
									get = function() return SelectedEncounter end,
									set = function(info,value) SelectedEncounter = value end,
									values = function()
										wipe(EncounterList)
										for k in addon:IterateEDB() do
											EncounterList[k] = addon.EDB[k].name
										end
										return EncounterList
									end,
								},
								DisableSelected = {
									order = 200,
									type = "execute",
									name = L.options["Set selected to None"],
									desc = L.options["Sets every alert's sound in the selected encounter to None"],
									disabled = function() return not SelectedEncounter end,
									confirm = true,
									func = function()
										for var,stgs in pairs(db.profile.Encounters[SelectedEncounter]) do
											if stgs.sound then stgs.sound = "None" end
										end
									end,
								},
								reset = {
									type = "execute",
									name = L.options["Reset"],
									order = 300,
									func = function()
										for var,stgs in pairs(db.profile.Encounters[SelectedEncounter]) do
											if stgs.sound then
												stgs.sound = addon.defaults.profile.Encounters[SelectedEncounter][var].sound
											end
										end
									end,
								},								
							},
						},
					},
			}
			
			sounds_args.soundsin_group = soundsin_group	
	end

	
	---------------------------------------------
	-- Raid Auras
	---------------------------------------------
	do
		local RaidAuras = addon.RaidAuras
		local handler = {}
			local colors = {}
			local colors2 = {}
			for k,c in pairs(addon.Media.Colors) do
				local hex = ("|cff%02x%02x%02x%s|r"):format(c.r*255,c.g*255,c.b*255,L[k])
				colors[k] = hex
				colors2[k] = hex
			end
			colors["GLOBAL"] = L.options["GLOBAL"]
			colors2["Off"] = OFF
			
			function handler:GetColor2(info)
				return colors2
			end			
			function handler:GetColor3(info)
				return colors
			end			
			
		local auras_anchor_group = {
			type = "group",
			name = L.options["Raid Auras"],
			order = 400,
			childGroups = "tab",
			args = {
				header = {
					type = "header",
					name = L.options["Raid Auras"],
					order = 1200,
				},
				desc = {
					type = "description",
					name = L.options["Gives timer bars and raid messages about common buffs when used (ex: tranquility) and raid utils (ex: feasts and jeeves)"],
					order = 1201,
					width = "full",
				},
			},
		}
		opts_args.auras_anchor_group = auras_anchor_group
		local auras_args = auras_anchor_group.args	
			
			
		local AU_group = {
			type = "group",
			name = L.options["Settings"],
			get = function(info) return RaidAuras.db.profile[info[#info]] end,
			set = function(info,v) RaidAuras.db.profile[info[#info]] = v end,
			handler = handler,
			order = 1,
			args = {
				blank1 = genblank(101),
				Enabled = {
					type = "toggle",
					name = L.options["Enable Raid Auras"],
					order = 1210,
					width = "normal",
						set = function(info,v)
							local key,var = info[3],info[5]
							RaidAuras.db.profile.Enabled = v
							RaidAuras.RefreshProfile()
						end,					
				},	
				Arrows = {
					type = "toggle",
					name = L.options["Arrows"],
					desc = L.options["Enable the display of arrows when a out of combat utility is cast, ex:"].."|cffffd200 "..GetSpellInfo(67826).."|h|r",
					order = 1212,
					width = "normal",
					set = function(info,v)
						local key,var = info[3],info[5]
						RaidAuras.db.profile.Arrows = v
						RaidAuras.RefreshProfile()
					end,
					disabled = function() return not RaidAuras.db.profile.Enabled end,
				},
				bars_group = {
					type = "group",
					name = L.options["Bars"],
					order = 1305,
					inline = true,
					args = {
						Bars = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1210,
							width = "half",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.Bars = v
								RaidAuras.RefreshProfile()
							end,
							disabled = function() return not RaidAuras.db.profile.Enabled end,
						},
						BarColor = {
							type = "select",
							name = L.options["Main Color"],
							order = 2000,
							values = "GetColor3",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.BarColor = v
								RaidAuras.RefreshProfile()
							end,
							disabled = function() return not RaidAuras.db.profile.Bars or not RaidAuras.db.profile.Enabled end,
						},
						Flashcolor = {
							type = "select",
							name = L.options["Flash Color"],
							order = 2001,
							values = "GetColor2",
							disabled = function() return not RaidAuras.db.profile.Bars or not RaidAuras.db.profile.Enabled end,
						},						
						TestBar = {
							type = "execute",
							name = L.options["Test Bars"],
							order = 2002,
							func = function(info)
								addon.Alerts:Auras("raidaurasTest", getclasscolor(UnitName("player"))..": "..addon.SN[740], 30, 15, "DXE ALERT2", addon.RaidAuras.db.profile.BarColor, "GREEN", nil, addon.ST[740])
							end,
							disabled = function() return not RaidAuras.db.profile.Bars or not RaidAuras.db.profile.Enabled end,
						},
					},
				},				
				message_group = {
					type = "group",
					name = L.options["Messages"],
					order = 1310,
					inline = true,
					args = {
						Messages = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1211,
							width = "half",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.Messages = v
								RaidAuras.RefreshProfile()
							end,
							disabled = function() return not RaidAuras.db.profile.Enabled end,
						},
						color1 = {
							type = "select",
							name = L.options["Messages Color"],
							order = 1250,
							values = "GetColor3",
							disabled = function() return not RaidAuras.db.profile.Messages or not RaidAuras.db.profile.Enabled end,
						},
						sound = {
							type = "select",
							name = L.options["Messages Sound"],
							order = 1280,
							values = GetSounds,
							disabled = function() return not RaidAuras.db.profile.Messages or not RaidAuras.db.profile.Enabled end,
						},
						TestMessage = {
							type = "execute",
							name = L.options["Test"],
							order = 2002,
							func = function(info)
								addon.Alerts:NewTMessage(getclasscolor(UnitName("player"))..": "..addon.SN[740],addon.RaidAuras.db.profile.color1,addon.ST[740],"ALERT13")
							end,
							disabled = function() return not RaidAuras.db.profile.Messages or not RaidAuras.db.profile.Enabled end,
						},
					},
				},
				misc_group = {
					type = "group",
					name = L.options["Chat Messages"],
					order = 1311,
					inline = true,
					args = {	
						desc = {
							type = "description",
	--						name = "|cffffd244          "..L.options["This option enables the display to raid or party when a aura is cast and when it ends, ex: Tranquility has started - Tranquility nas ended."],
							name = L.options["This enables the display to raid or party when a aura is cast and when it ends, ex: |cffffd200Tranquility activated! (8s)|h|r and |cffffd200Tranquility has ended!|h|r"],
							order = 121,
							width = "full",
						},						
						blank = genblank(121.5),
						blank2 = genblank(121.6),							
						ChatMessages = {
							type = "toggle",
							name = L.options["Enable"],
							order = 1215,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.ChatMessages = v
								RaidAuras.RefreshProfile()
							end,
							disabled = function() return not RaidAuras.db.profile.Enabled end,
						},
						blank3 = genblank(1216.5),
						blank4 = genblank(1216.6),
						desc2 = {
							type = "description",
							name = L.options["Here you can choose if the raid or party message is just for your activated auras or all players activated auras"],
							order = 1217,
							width = "full",
						},	
						blank5 = genblank(1217.5),
						blank6 = genblank(1217.6),					
						OnlyPlayer = {
							type = "toggle",
							name = L.options["Just Yours"],
							order = 1218,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.OnlyPlayer = v
								if v == true then 
									RaidAuras.db.profile.AllPlayers = false
									--RaidAuras.db.profile.PrintMsg = false
								else 
									RaidAuras.db.profile.AllPlayers = true 
									--RaidAuras.db.profile.PrintMsg = true 
									--RaidAuras.db.profile.ChatMessages = false
								end
								RaidAuras.RefreshProfile()
							end,
							disabled = function() return not RaidAuras.db.profile.ChatMessages or not RaidAuras.db.profile.Enabled end,						
						},
						AllPlayers = {
							type = "toggle",
							name = L.options["All"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.AllPlayers = v
								if v == true then 
									RaidAuras.db.profile.OnlyPlayer = false
									--RaidAuras.db.profile.PrintMsg = false
								else 
									RaidAuras.db.profile.OnlyPlayer = true 
									--RaidAuras.db.profile.PrintMsg = true 
									--RaidAuras.db.profile.ChatMessages = false
								end									
								RaidAuras.RefreshProfile()
							end,	
							disabled = function() return not RaidAuras.db.profile.ChatMessages or not RaidAuras.db.profile.Enabled end,
						},
						PrintMsg = {
							type = "toggle",
							name = L.options["Local Message Only"],
							order = 1219,
							width = "normal",
							set = function(info,v)
								local key,var = info[3],info[5]
								RaidAuras.db.profile.PrintMsg = v
								--if v == true then 
									--RaidAuras.db.profile.OnlyPlayer = false
									--RaidAuras.db.profile.AllPlayers = false
								--else RaidAuras.db.profile.OnlyPlayer = true end									
								RaidAuras.RefreshProfile()
							end,	
							disabled = function() return not RaidAuras.db.profile.ChatMessages or not RaidAuras.db.profile.Enabled end,
						},						
					},
				},
				exclude_group = {
					type = "group",
					name = L.options["Exclude"],
					order = 1312,
					inline = true,
					args = {	
						exhealer = {
							type = "toggle",
							name = L.options["Healers"],
							width = "half",
							order = 1,
						},
						extank = {
							type = "toggle",
							name = L.options["Tanks"],
							width = "half",
							order = 2,
						},
						exdps = {
							type = "toggle",
							name = L.options["Dps"],
							width = "half",
							order = 3,
						},
					},
				},
			},
		}

		auras_args.AU_group = AU_group
		
		local auras2_group = {
			type = "group",
			name = L.options["Auras"].." & "..L.options["Utils"],
			get = function(info) return RaidAuras.db.profile[info[#info]] end,
			set = function(info,v) RaidAuras.db.profile[info[#info]] = v end,
			handler = handler,
			order = 2,
			args = {
				header2 = {
					type = "header",
					name = L.options["Utils"],
					order = 1214,
				},	
				Feasts = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(160740))..":18:18:-5|t"..GetSpellInfo(160740),
					desc = GetSpellDescription(160740),
					order = 1215,
					width = "double",
				},
				Repair = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(67826))..":18:18:-5|t"..GetSpellInfo(67826),
					desc = GetSpellDescription(67826),
					order = 1216,
					width = "double",
				},
				Soulwell = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(29893))..":18:18:-5|t"..GetSpellInfo(29893),
					desc = GetSpellDescription(29893),
					order = 1217,
					width = "double",
				},
				SummoningStone = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(698))..":18:18:-5|t"..GetSpellInfo(698),
					desc = GetSpellDescription(698),
					order = 1218,
					width = "double",
				},					
				header3 = {
					type = "header",
					name = L.options["Auras"],
					order = 1300,
				},
				RallyingCry = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(97462))..":18:18:-5|t"..GetSpellInfo(97462),
					desc = GetSpellDescription(97462),
					order = 1600,
					width = "double",
				},
				StampedingRoar = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(106898))..":18:18:-5|t"..GetSpellInfo(106898),
					desc = GetSpellDescription(106898),
					order = 1601,
					width = "double",
				},
				SmokeBomb = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(76577))..":18:18:-5|t"..GetSpellInfo(76577),
					desc = GetSpellDescription(76577),
					order = 1603,
					width = "double",
				},	
				PainSuppression = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(33206))..":18:18:-5|t"..GetSpellInfo(33206),
					desc = GetSpellDescription(33206),
					order = 1604,
					width = "double",
				},
				GuardianSpirit = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(47788))..":18:18:-5|t"..GetSpellInfo(47788),
					desc = GetSpellDescription(47788),
					order = 1605,
					width = "double",
				},	
				Barrier = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(62618))..":18:18:-5|t"..GetSpellInfo(62618),
					desc = GetSpellDescription(62618),
					order = 1606,
					width = "double",
				},	
				DivineHymn = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(64843))..":18:18:-5|t"..GetSpellInfo(64843),
					desc = GetSpellDescription(64843),
					order = 1607,
					width = "double",
				},	
				Tranquility = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(740))..":18:18:-5|t"..GetSpellInfo(740),
					desc = GetSpellDescription(740),
					order = 1608,
					width = "double",
				},
				Ironbark = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(102342))..":18:18:-5|t"..GetSpellInfo(102342),
					desc = GetSpellDescription(102342),
					order = 1609,
					width = "double",
				},
				Sacrifice = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(6940))..":18:18:-5|t"..GetSpellInfo(6940),
					desc = GetSpellDescription(6940),
					order = 1610,
					width = "double",
				},
				DevotionAura = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(31821))..":18:18:-5|t"..GetSpellInfo(31821),
					desc = GetSpellDescription(31821),
					order = 1611,
					width = "double",
				},	
				SpiritLink = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(98008))..":18:18:-5|t"..GetSpellInfo(98008),
					desc = GetSpellDescription(98008),
					order = 1612,
					width = "double",
				},
				LifeCocoon = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(116849))..":18:18:-5|t"..GetSpellInfo(116849),
					desc = GetSpellDescription(116849),
					order = 1613,
					width = "double",
				},
				Revival = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(115310))..":18:18:-5|t"..GetSpellInfo(115310),
					desc = GetSpellDescription(115310),
					order = 1614,
					width = "double",
				},
				HealingTide = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(108280))..":18:18:-5|t"..GetSpellInfo(108280),
					desc = GetSpellDescription(108280),
					order = 1614.5,
					width = "double",
 				},				
				AntiMagicZone = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(51052))..":18:18:-5|t"..GetSpellInfo(51052),
					desc = GetSpellDescription(51052),
					order = 1615,
					width = "double",
				},
				Rebirth = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(20484))..":18:18:-5|t"..GetSpellInfo(20484),
					desc = GetSpellDescription(20484),
					order = 1617,
					width = "double",
				},
				Bloodlust = {
					type = "toggle",
					name =  "  |T"..select(3,GetSpellInfo(2825))..":18:18:-5|t"..GetSpellInfo(2825),
					desc = GetSpellDescription(2825),
					order = 1618,
					width = "double",
				},
			},
		 }
		 auras_args.auras2_group = auras2_group

	end	
	
	---------------------------------------------
	-- Slash Commands
	---------------------------------------------

	do
		local loadselect
		local handler = {}
		local slashcmd_group = {
			type = "group",
			name = L.options["Slash Commands"],
			order = -3,
			childGroups = "tab",
			handler = handler,
			args = {
				SlashTitle = {
					type = "header",
					name = L.options["|cFFFED000Slash Commands:|r"],
					--fontSize = "large",
					order = 1320,
					--width = "full",
				},
				SlashDxeEnable = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxe enable:|r Enable addon"],
					fontSize = "medium",
					order = 1321,
					width = "full",
				},	
				SlashDxeDisable = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxe disable:|r Disable addon"],
					fontSize = "medium",
					order = 1322,
					width = "full",
				},	
				SlashDxeConfig = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxe config:|r Toggles configuration"],
					fontSize = "medium",
					order = 1323,
					width = "full",
				},	
				SlashDxeVersion = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxe version:|r Show version check window"],
					fontSize = "medium",
					order = 1324,
					width = "full",
				},	
				SlashLPull = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/lpull time:|r (local bar) Creates a local pull bar that only you can see"],
					fontSize = "medium",
					order = 1324.5,
					width = "full",
				},					
				SlashPull = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/pull time:|r (raid bar) Sends a pull countdown to your raid"],
					fontSize = "medium",
					order = 1325,
					width = "full",
				},	
				SlashLBreak = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/lbreak time text:|r (local bar) Creates a local break bar that only you can see"],
					fontSize = "medium",
					order = 1326,
					width = "full",
				},	
				SlashRBreak = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/break time text:|r (raid bar) Sends a break countdown to your raid"],				
					fontSize = "medium",
					order = 1327,
					width = "full",
				},
				CustomLBreak = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxelb time text:|r (local bar) Creates a local break bar that only you can see"],
					fontSize = "medium",
					order = 1328,
					width = "full",
				},	
				CustomRBreak = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/dxerb time text:|r (raid bar) Sends a custom bar to your raid"],				
					fontSize = "medium",
					order = 1329,
					width = "full",
				},
				SlashRange = {
					type = "description",
					name = "\n    ".. L.options["|cFFFED000/range yards:|r Open a proximity window with choosen yards"],
					fontSize = "medium",
					order = 1330,
					width = "full",
				},
				SlashDesc = {
					type = "description",
					name = "\n\n             ".. L.options["|cFFFED000time|r can be in the format |cFFFED000minutes:seconds|r or |cFFFED000seconds|r"],				
					fontSize = "large",
					order = 1331,
					width = "full",
				},				
			},
		}
		opts_args.slashcmd_group = slashcmd_group	
	end	
	---------------------------------------------
	-- DEBUG
	---------------------------------------------

	--[===[@debug@
	local debug_group = {
		type = "group",
		order = -2,
		name = "Debug",
		args = {},
	}
	opts_args.debug_group = debug_group
	addon:AddDebugOptions(debug_group.args)
	--@end-debug@]===]

end

---------------------------------------------
-- INITIALIZATION
---------------------------------------------

function module:OnInitialize()
	db = addon.db
	InitializeOptions()
	InitializeOptions = nil
	self:FillEncounters()

	opts_args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
	if addon.LDS then addon.LDS:EnhanceOptions(opts_args.profile,db) end
	opts_args.profile.order = -10

	AC:RegisterOptionsTable("DXE", opts)
	ACD:SetDefaultSize("DXE", DEFAULT_WIDTH, DEFAULT_HEIGHT)
end

function module:ToggleConfig() ACD[ACD.OpenFrames.DXE and "Close" or "Open"](ACD,"DXE") end
